/*
@copyright(disclaimer)

Licensed Materials - Property of IBM
IBM DB2 Content Manager Enterprise Edition V8  (program number 5724-B19)
IBM DB2 Content Manager Express Edition V8 (program number 5724-F73)
IBM DB2 Content Manager for z/OS V8 (program number 5697-H60)
(c ) Copyright IBM Corp. 2005. All Rights Reserved.

US Government Users Restricted Rights
Use, duplication or disclosure restricted by GSA ADP Schedule
Contract with IBM Corp.

DISCLAIMER OF WARRANTIES :

Permission is granted to copy and modify this  Sample code, and to
distribute modified versions provided that both the copyright
notice,- and this permission notice and warranty disclaimer  appear in all copies and modified
versions.

THIS SAMPLE CODE IS LICENSED TO YOU AS-IS.
IBM  AND ITS SUPPLIERS AND LICENSORS  DISCLAIM
ALL WARRANTIES, EITHER EXPRESS OR IMPLIED, IN SUCH SAMPLE CODE, INCLUDING THE WARRANTY
OF NON-INFRINGEMENT AND THE IMPLIED WARRANTIES OF MERCHANTABILITY OR FITNESS FOR A
PARTICULAR PURPOSE. IN NO EVENT WILL IBM OR ITS LICENSORS OR SUPPLIERS BE LIABLE FOR ANY
DAMAGES ARISING OUT OF THE USE OF  OR INABILITY TO USE THE SAMPLE CODE, DISTRIBUTION OF
THE SAMPLE CODE, OR COMBINATION OF THE SAMPLE CODE WITH ANY OTHER CODE. IN NO EVENT SHALL
IBM OR ITS LICENSORS AND SUPPLIERS BE LIABLE FOR ANY LOST REVENUE, LOST PROFITS OR DATA,
OR FOR DIRECT, INDIRECT, SPECIAL, CONSEQUENTIAL,INCIDENTAL OR PUNITIVE DAMAGES, HOWEVER
CAUSED AND REGARDLESS OF THE THEORY OF LIABILITY,-, EVEN IF IBM OR ITS LICENSORS OR SUPPLIERS
HAVE BEEN ADVISED OF THE POSSIBILITY OF SUCH DAMAGES.

@endCopyright
*/

package com.ibm.bpe.cm.sca;

import java.io.Serializable;
import java.util.ArrayList;
import java.util.List;
import java.util.Map;

import com.ibm.bpe.cm.CMBPCConstants;
import com.ibm.bpe.cm.api.CMBPCIntegration;
import com.ibm.bpe.cm.util.CombinedQueryResultSet;
import com.ibm.websphere.bo.BOFactory;
import com.ibm.websphere.sca.ServiceManager;
import commonj.sdo.DataObject;

/**
 * Implementation of the SCA component CMServices providing the following operations:
 * 
 * evaluateCondition, getContentAttribute, getFolderContentAttribute
 * getContentAttributes, getFolderContentAttributes
 * combinedQuery, addItemToFlowFolder, addFlowItemToFolder
 * reindex, setACL, createItemRef
 * 
 */
public class CMServicesImpl implements Serializable {

	/**
	 * Default constructor.
	 */
	public CMServicesImpl() {
		super();
	}

	/**
	 * Return a reference to the component service instance for this implementation
	 * class.  This method should be used when passing this service to a partner reference
	 * or if you want to invoke this component service asynchronously.    
	 *
	 * @generated (com.ibm.wbit.java)
	 */
	private Object getMyService() {
		return (Object) ServiceManager.INSTANCE.locateService("self");
	}

	/**
	* Internal function to cache the Business Object Factory
	*/
	static private BOFactory _bof = null;

	/**
	 * Internal function to cache the Business Object Factory
	 * @return business object factory
	 */
	static private BOFactory getBOF() {
		if (_bof == null)_bof = (BOFactory) ServiceManager.INSTANCE.locateService(CMBPCConstants.BO_FACTORY);
		return _bof;
	}

   /**
	* Evaluate a Content Manager XPath expression.
	* Use this function in a Process Choreographer transition condition to
	* determine a flow route based on document contents.
	* @param xPathQueryString A Content Manager XPath expression for a condition
	* @return true, if the condition is true
	*/
	public Boolean evaluateCondition(String xPathQueryString) {
		boolean result = CMBPCIntegration.evaluateCondition(xPathQueryString);
		return new Boolean(result);
	}

   /**
  	* Get an attribute value of the associated item (when it is no folder).
  	* @param itemRef - reference to the item that is routed through the process
  	* @param attributeSpec - Attribute to retrieve in &lt;item-type&gt;/&lt;attribute-name&gt; 
  	* format, e.g. <code>"XYZ_ClaimForm/XYZ_DriversLic"</code>.
  	* Note that &lt;attribute-name&gt; can include child component names,
  	* e.g. Book/Chapter/Section/Author.
  	* @return an Object for the attribute value or null if the attribute cannot be resolved.
  	*/
	public DataObject getContentAttribute(DataObject itemRef,
										  String attributeSpec) {
		DataObject result = null;
       	String pidString = itemRef.getString(CMBPCConstants.BOATT_ITEM_REF_PIDSTRING);
       	Object value = CMBPCIntegration.getContentAttribute(pidString, attributeSpec);
       	BOFactory bof = getBOF();
		if (bof != null) {
			result = bof.create(CMBPCConstants.LIB_NAMESPACE, CMBPCConstants.BO_ATTRIBUTE_VALUE);
			result.set(CMBPCConstants.BOATT_ATTRIBUTE_VALUE_VALUE, value);
		}
		return result;
	}

   /**
    * Get the attributes of of the specified item.
  	* @param itemRef - reference to the item that is routed through the process
	* @param attributeSpec - list of attributes to retrieve in &lt;item-type&gt;/&lt;attribute-name&gt; format, e.g.
	* <code><pre>
	* {
	*     "XYZ_ClaimForm/XYZ_DriversLic",
	*     "XYZ_ClaimForm/XYZ_PolicyNum",
	*     "XYZ_AdjReport/XYZ_LicPlate"
	* }
	* </pre></code>
	* Note that &lt;attribute-name&gt; can include child component names,
	* e.g. Book/Chapter/Section/Author.
	* @return an ArrayList of ArrayLists (e.g. a table) containing
	* one row for each returned attribute and columns for item type
	* name, item pid string, attribute name and attribute value.
	*/
	public DataObject getContentAttributes(DataObject itemRef,
										   DataObject attributeSpec) {
       	String pidString = itemRef.getString(CMBPCConstants.BOATT_ITEM_REF_PIDSTRING);
       	List attributeSpecList = attributeSpec.getList(CMBPCConstants.BOATT_ATTRIBUTE_SPECS_SPECS);
       	String[] attributeSpecArray = (String[])attributeSpecList.toArray();
       	Map valueMap = CMBPCIntegration.getContentAttributes(pidString, attributeSpecArray);
       	return createAttributesBO(attributeSpecArray, valueMap);
	}

   /**
  	* Get an attribute value of items in the specified folder.
  	* This is useful when a CM folder is started on a WPC
  	* process, and the process needs to get the value of
  	* attributes of items (e.g. the license plate attribute
  	* in the adjuster's report item type).
  	* @param itemRef - reference to the item that is routed through the process
  	* @param attributeSpec - Attribute to retrieve in &lt;item-type&gt;/&lt;attribute-name&gt; 
  	* format, e.g. <code>"XYZ_ClaimForm/XYZ_DriversLic"</code>.
  	* Note that &lt;attribute-name&gt; can include child component names,
  	* e.g. Book/Chapter/Section/Author.
  	* @return an Object for the attribute value or null if the attribute cannot be resolved.		
  	*/
	public DataObject getFolderContentAttribute(DataObject itemRef,
					  							String attributeSpec) {
		DataObject result = null;
       	String pidString = itemRef.getString(CMBPCConstants.BOATT_ITEM_REF_PIDSTRING);
       	Object value = CMBPCIntegration.getFolderContentAttribute(pidString, attributeSpec);
       	BOFactory bof = getBOF();
		if (bof != null) {
			result = bof.create(CMBPCConstants.LIB_NAMESPACE, CMBPCConstants.BO_ATTRIBUTE_VALUE);
			result.set(CMBPCConstants.BOATT_ATTRIBUTE_VALUE_VALUE, value);
		}
		return result;
	}

   /**
    * Get the attributes of items in the specified folder.
    * This is useful when a CM folder is started on a WPC
    * process, and the process needs to get the value of
    * attributes of items (e.g. the license plate attribute
    * in the adjuster's report item type).
  	* @param itemRef - reference to the item that is routed through the process
    * @param attributeSpec - list of attributes to retrieve in &lt;item-type&gt;/&lt;attribute-name&gt; 
    * format, e.g.
    * <code><pre>
    * {
    *     "XYZ_ClaimForm/XYZ_DriversLic",
    *     "XYZ_ClaimForm/XYZ_PolicyNum",
    *     "XYZ_AdjReport/XYZ_LicPlate"
    * }
    * </pre></code>
    * Note that &lt;attribute-name&gt; can include child component names,
    * e.g. Book/Chapter/Section/Author.
    * @return an ArrayList of ArrayLists (e.g. a table) containing
    * one row for each returned attribute and columns for item type
    * name, item pid string, attribute name and attribute value.
    */
	public DataObject getFolderContentAttributes(DataObject itemRef,
												 DataObject attributeSpec) {
       	String pidString = itemRef.getString(CMBPCConstants.BOATT_ITEM_REF_PIDSTRING);
       	List attributeSpecList = attributeSpec.getList(CMBPCConstants.BOATT_ATTRIBUTE_SPECS_SPECS);
       	String[] attributeSpecArray = (String[])attributeSpecList.toArray();
       	Map valueMap = CMBPCIntegration.getFolderContentAttributes(pidString, attributeSpecArray);
       	return createAttributesBO(attributeSpecArray, valueMap);
 	}

	/**
	 * Create a Business Object of type "AttributeValue" that contains values for the attribute specs
	 * at the corresponding index position of the attributeSpecArray.
	 * @param attributeSpecArray array of attribute specifications 
	 * @param valueMap maps attribute specs to the corresponding values (null if they do not exist)
	 * @return a Data Object of type "AttributeValue" 
	 */
	private DataObject createAttributesBO(String[] attributeSpecArray,
										  Map valueMap) {
		
		DataObject bo = null;
       	BOFactory bof = getBOF();
		if (bof != null) {
			int numValues = attributeSpecArray.length;
	       	ArrayList valueObjects = new ArrayList(numValues);
	       	Object value;
	       	for (int i = 0; i < numValues; i++) {
	       		value = valueMap.get(attributeSpecArray[i]);
	       		bo = bof.create(CMBPCConstants.LIB_NAMESPACE, CMBPCConstants.BO_ATTRIBUTE_VALUE);
	       		bo.set(CMBPCConstants.BOATT_ATTRIBUTE_VALUE_VALUE, value);
	       		valueObjects.add(bo);
	       	}
			bo = bof.create(CMBPCConstants.LIB_NAMESPACE, CMBPCConstants.BO_ATTRIBUTE_VALUES);
			bo.set(CMBPCConstants.BOATT_ATTRIBUTE_VALUES_VALUES, valueObjects);
		}
		return bo;
	}
	
	/**
	 * Method generated to support implemention of operation "combinedQuery" defined for WSDL port type 
	 * named "interface.CMServicesInterface".
	 * 
	 * The presence of commonj.sdo.DataObject as the return type and/or as a parameter 
	 * type conveys that its a complex type. Please refer to the WSDL Definition for more information 
	 * on the type of input, output and fault(s).
	 * 
	 * @see com.ibm.bpe.cm.util.CombinedQuery for more details
	 */
	public DataObject combinedQuery(String selectClauseWorkflow,
			String flowItemTypeNameContent,
			String flowItemTypeAttributesContent,
			String additionalAttributesContent,
			String whereClauseWorkflow,
			String xPathContent,
			String orderByClauseWorkflow,
			String timeZoneWorkflowString) {

		CombinedQueryResultSet cqrs = CMBPCIntegration.combinedQuery(selectClauseWorkflow,
												  flowItemTypeNameContent, flowItemTypeAttributesContent,
												  additionalAttributesContent, whereClauseWorkflow,
												  xPathContent, orderByClauseWorkflow, timeZoneWorkflowString);
		return cqrs.getDataObject();
	}

   /**
	* Add the results of a Content Manager query to the folder that is routed through the process.
	* 
  	* @param itemRef - reference to the item that is routed through the process
	* @param query XPath query string that specifies the items to be added
	* @return number of items that have been added to the folder (number of hits); -1 in case of failure
	*/
	public Integer addItemToFlowFolder(DataObject itemRef,
									   String xPathQueryString) {
       	String pidString = itemRef.getString(CMBPCConstants.BOATT_ITEM_REF_PIDSTRING);
       	int result = CMBPCIntegration.addItemToFlowFolder(pidString, xPathQueryString);
		return new Integer(result);
	}

   /**
    * Add the item routed through the process to the folder retrieved by a Content Manager query. 
    * 
  	* @param itemRef - reference to the item that is routed through the process
    * @param query XPath query string that specifies the folder to which this item should be added
    * @return number of folders that matched the query; -1 in case of failure
    */
	public Integer addFlowItemToFolder(DataObject itemRef,
									   String xPathQueryString) {
       	String pidString = itemRef.getString(CMBPCConstants.BOATT_ITEM_REF_PIDSTRING);
       	int result = CMBPCIntegration.addFlowItemToFolder(pidString, xPathQueryString);
		return new Integer(result);
	}

   /**
    * Assign the item routed through the process to a new item type.
    * Note: this changes the itemPid so it needs to be reassigned to the process
    * property when reindex() has been called to ensure the client displays
    * the appropriate set of attributes/parts.
    *   
  	* @param itemRef - reference to the item that is routed through the process
    * @param newItemType name of the new item type the item should be assigned to
    */
	public void reindex(DataObject itemRef,
						String newItemType) {
       	String pidString = itemRef.getString(CMBPCConstants.BOATT_ITEM_REF_PIDSTRING);
       	CMBPCIntegration.reindex(pidString, newItemType);
	}

   /**
	* Set the ACL of an item to a new value. Note that this requires item-level ACL checking.
	* 
  	* @param itemRef - reference to the item that is routed through the process
	* @param newACL - ACL to be assigned to this item
	*/
	  
	public void setACL(DataObject itemRef, String newACL) {
       	String pidString = itemRef.getString(CMBPCConstants.BOATT_ITEM_REF_PIDSTRING);
       	CMBPCIntegration.setACL(pidString, newACL);
	}
	
	
	/**
	 * Create an itemPid from itemId, componentId, versionNo. Note that the returned itemPid also contains the 
	 * item type which is needed e.g. by the QuickStartClient. This operation needs to be called whenever an
	 * item is moved to a new type to update the ItemRefs used in the process.
	 * 
	 * @param itemId id of this item
	 * @param componentId component id of this item
	 * @param versionNo version number of this item
	 * @return fully specified itemPid corresponding to to this itemId/componentId/versionNo including server name, type and item type
	 */

	public DataObject createItemRef(String itemId,
					                String componentId,
								    Integer versionNo) {
		DataObject result = null;
		String pidString = CMBPCIntegration.getPid(itemId, componentId, versionNo);
       	BOFactory bof = getBOF();
		if (bof != null) {
			result = bof.create(CMBPCConstants.LIB_NAMESPACE, CMBPCConstants.BO_ITEM_REF);
			result.setString(CMBPCConstants.BOATT_ITEM_REF_PIDSTRING, pidString);
		}
		return result;
	}

}


