/*
@copyright(disclaimer)

Licensed Materials - Property of IBM
IBM DB2 Content Manager Enterprise Edition V8  (program number 5724-B19)
IBM DB2 Content Manager Express Edition V8 (program number 5724-F73)
IBM DB2 Content Manager for z/OS V8 (program number 5697-H60)
(c ) Copyright IBM Corp. 2005. All Rights Reserved.

US Government Users Restricted Rights
Use, duplication or disclosure restricted by GSA ADP Schedule
Contract with IBM Corp.

DISCLAIMER OF WARRANTIES :

Permission is granted to copy and modify this  Sample code, and to
distribute modified versions provided that both the copyright
notice,- and this permission notice and warranty disclaimer  appear in all copies and modified
versions.

THIS SAMPLE CODE IS LICENSED TO YOU AS-IS.
IBM  AND ITS SUPPLIERS AND LICENSORS  DISCLAIM
ALL WARRANTIES, EITHER EXPRESS OR IMPLIED, IN SUCH SAMPLE CODE, INCLUDING THE WARRANTY
OF NON-INFRINGEMENT AND THE IMPLIED WARRANTIES OF MERCHANTABILITY OR FITNESS FOR A
PARTICULAR PURPOSE. IN NO EVENT WILL IBM OR ITS LICENSORS OR SUPPLIERS BE LIABLE FOR ANY
DAMAGES ARISING OUT OF THE USE OF  OR INABILITY TO USE THE SAMPLE CODE, DISTRIBUTION OF
THE SAMPLE CODE, OR COMBINATION OF THE SAMPLE CODE WITH ANY OTHER CODE. IN NO EVENT SHALL
IBM OR ITS LICENSORS AND SUPPLIERS BE LIABLE FOR ANY LOST REVENUE, LOST PROFITS OR DATA,
OR FOR DIRECT, INDIRECT, SPECIAL, CONSEQUENTIAL,INCIDENTAL OR PUNITIVE DAMAGES, HOWEVER
CAUSED AND REGARDLESS OF THE THEORY OF LIABILITY,-, EVEN IF IBM OR ITS LICENSORS OR SUPPLIERS
HAVE BEEN ADVISED OF THE POSSIBILITY OF SUCH DAMAGES.

@endCopyright
*/

package com.ibm.bpe.cm.util;

import java.io.Serializable;

import com.ibm.bpe.cm.CMBPCConstants;
import com.ibm.websphere.bo.BOFactory;
import com.ibm.websphere.sca.ServiceCallback;
import com.ibm.websphere.sca.ServiceManager;
import com.ibm.websphere.sca.Ticket;
import commonj.sdo.DataObject;

/**
 * This bean class keeps all information required to query for collection
 * points and to send a message to the replyQ once the collection
 * point completed. 
 */
public class CollectionPointCallbackInfo implements Serializable
{

  /**
   * Create a new collection point definition.
   * @param correlation The JMS correlation ID to send the answer back to the service
   * @param folderItemId The ID of the folder
   * @param itemTypeNames The list of item type names to wait for
   * @param itemTypeQuantities The list of item type quantities to wait for. Size must match with itemTypeNames
   * @param timeout The UTC time in millis when the collection point "expires". A value of "0" means that
   * it is checked "forever".
   */
  public CollectionPointCallbackInfo(ServiceCallback serviceCallback,
  									 Ticket ticket)
  {
    _serviceCallback = serviceCallback;
    _ticket = ticket;
  }
  
  /**
   * Inform the caller that the collection point condition is satisfied
   * @param result of the collection point
   */
  public void notifyBusinessProcess(int result)
  {
  	/**
	 * Inform the caller that the collection point condition is satisfied
	 */
	try
	{
		BOFactory bof = (BOFactory) ServiceManager.INSTANCE.locateService(CMBPCConstants.BO_FACTORY);		    
	    DataObject bo = bof.createByElement(CMBPCConstants.CMCOLLECTIONPOINTSERVICE_NAMESPACE, CMBPCConstants.BOATT_COLLECTIONPOINT_RESPONSE);		    
	    bo.setInt(CMBPCConstants.BOATT_COLLECTIONPOINT_RESPONSE_RESULT, result);		    
		_serviceCallback.onInvokeResponse(_ticket, bo, null);		
	}
	catch( Throwable exc )
	{
		exc.printStackTrace();
	}	  	  	
  }  
  
  private ServiceCallback _serviceCallback = null;
  private Ticket _ticket = null;

}
