/*
@copyright(disclaimer)

Licensed Materials - Property of IBM
IBM DB2 Content Manager Enterprise Edition V8  (program number 5724-B19)
IBM DB2 Content Manager Express Edition V8 (program number 5724-F73)
IBM DB2 Content Manager for z/OS V8 (program number 5697-H60)
(c ) Copyright IBM Corp. 2005. All Rights Reserved.

US Government Users Restricted Rights
Use, duplication or disclosure restricted by GSA ADP Schedule
Contract with IBM Corp.

DISCLAIMER OF WARRANTIES :

Permission is granted to copy and modify this  Sample code, and to
distribute modified versions provided that both the copyright
notice,- and this permission notice and warranty disclaimer  appear in all copies and modified
versions.

THIS SAMPLE CODE IS LICENSED TO YOU AS-IS.
IBM  AND ITS SUPPLIERS AND LICENSORS  DISCLAIM
ALL WARRANTIES, EITHER EXPRESS OR IMPLIED, IN SUCH SAMPLE CODE, INCLUDING THE WARRANTY
OF NON-INFRINGEMENT AND THE IMPLIED WARRANTIES OF MERCHANTABILITY OR FITNESS FOR A
PARTICULAR PURPOSE. IN NO EVENT WILL IBM OR ITS LICENSORS OR SUPPLIERS BE LIABLE FOR ANY
DAMAGES ARISING OUT OF THE USE OF  OR INABILITY TO USE THE SAMPLE CODE, DISTRIBUTION OF
THE SAMPLE CODE, OR COMBINATION OF THE SAMPLE CODE WITH ANY OTHER CODE. IN NO EVENT SHALL
IBM OR ITS LICENSORS AND SUPPLIERS BE LIABLE FOR ANY LOST REVENUE, LOST PROFITS OR DATA,
OR FOR DIRECT, INDIRECT, SPECIAL, CONSEQUENTIAL,INCIDENTAL OR PUNITIVE DAMAGES, HOWEVER
CAUSED AND REGARDLESS OF THE THEORY OF LIABILITY,-, EVEN IF IBM OR ITS LICENSORS OR SUPPLIERS
HAVE BEEN ADVISED OF THE POSSIBILITY OF SUCH DAMAGES.

@endCopyright
*/

package com.ibm.bpe.cm.util;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

import javax.naming.InitialContext;

import com.ibm.bpe.cm.CMBPCConstants;
import com.ibm.bpe.cm.conmgmt.ManageConnection;
import com.ibm.mm.sdk.common.DKAttrDefICM;
import com.ibm.mm.sdk.common.DKComponentTypeDefICM;
import com.ibm.mm.sdk.common.DKConstant;
import com.ibm.mm.sdk.common.DKConstantICM;
import com.ibm.mm.sdk.common.DKDDO;
import com.ibm.mm.sdk.common.DKDatastoreDefICM;
import com.ibm.mm.sdk.common.DKFolder;
import com.ibm.mm.sdk.common.DKItemTypeDefICM;
import com.ibm.mm.sdk.common.DKNVPair;
import com.ibm.mm.sdk.common.DKPidICM;
import com.ibm.mm.sdk.common.DKResults;
import com.ibm.mm.sdk.common.DKSequentialCollection;
import com.ibm.mm.sdk.common.DKUsageError;
import com.ibm.mm.sdk.common.dkIterator;
import com.ibm.mm.sdk.common.dkResultSetCursor;
import com.ibm.mm.sdk.server.DKDatastoreICM;
import com.ibm.ws.extensionhelper.ExtensionHelper;
import com.ibm.ws.extensionhelper.TransactionControl;
import com.ibm.ws.extensionhelper.TxHandle;

/**
 * Functions to access the Content Manager API.
 * This includes static functions to connect / disconnect to CM, read the
 * settings from bpecm.properties and get document content for a given itemPid.
 */
public class ContentManagerAccess
{
	
  /**
   * Create a Content Manager access object.
   */
  public ContentManagerAccess()
  {
    // Initialize TX handling
    try
    {
      InitialContext initialContext = new InitialContext();
      extHelper = (ExtensionHelper) initialContext.lookup(CMBPCConstants.EXTENSION_HELPER_REF);
      tranControl = extHelper.getTransactionControl();
    }
    catch (Exception e)
    {
      // no error handling, as the WebSphere's emergency behaviour in this case is robust enough
    }
  }
 	  

  /**
   * Suspend an existing global transaction.
   * @return The transaction object. Use this object to resume the transaction via <code>resumeGlobalTransaction</code>.
   */
  protected TxHandle suspendGlobalTransaction()
  {
    TxHandle transaction = null;
    try
    {
      transaction = tranControl.preinvoke(true, false);
    }
    catch (javax.transaction.SystemException exc)
    {
      // Here, we can either continue processing or rethrow the exception.
      // Assuming that following JDBC calls may be successful even if the above exception was
      // thrown, we continue processing.
    }
    catch (javax.transaction.NotSupportedException exc)
    {
      // Here, we can either continue processing or rethrow the exception.
      // Assuming that following JDBC calls may be successful even if the above exception was
      // thrown, we continue processing.
    }
    return transaction;
  }

  /**
   * Resume a previously suspended global transaction.
   * @param transaction The transaction object returned by <code>suspendGlobalTransaction()</code>.
   */
  protected void resumeGlobalTransaction(final TxHandle transaction)
  {
    try
    {
      tranControl.postinvoke(transaction); // resumes previous global or local UOW
    }
    catch (javax.transaction.SystemException exc)
    {
      // Here, we can either continue processing or rethrow the exception.
      // Assuming that following JDBC calls may be successful even if the above exception was
      // thrown, we continue processing.
    }
    catch (javax.transaction.InvalidTransactionException exc)
    {
      // This exception is considered more severe than the SystemException during suspend above,
      // since we retrieved a non-null transaction object that now turns out to be invalid.
      throw IntegrationException.wrap(exc);
    }
    catch (com.ibm.ws.extensionhelper.exception.InconsistentTransactionException exc)
    {
      // This exception is considered more severe than the SystemException during suspend above,
      // since we retrieved a non-null transaction object that now turns out to be invalid.
      throw IntegrationException.wrap(exc);
    }
  }

  /**
   * For the given parts of an ID, construct an itemPID String and retrieve the corresponding DDO
   * to ensure the PID string contains all relevant information (including the item type id, data store, ...)
   * @param itemId
   * @param componentId
   * @param versionId
   * @return An itemPID string.
   */

  public String getPid(String itemId,
  					   String componentId,
  					   int versionNo)
  {
	DKDatastoreICM dkDatastore = null;
	String pidString = null;
	try
	{
	  dkDatastore = ManageConnection.connect();
	  String qs =
		"/*[@ITEMID=\""
		  + itemId
		  + "\" AND @COMPONENTID=\""
		  + componentId
		  + "\" AND @VERSIONID="
		  + versionNo
		  + "]";
	  DKNVPair options[] = new DKNVPair[2];
	  options[0] =
		new DKNVPair(DKConstant.DK_CM_PARM_RETRIEVE, new Integer(DKConstant.DK_CM_CONTENT_IDONLY));
	  options[1] = new DKNVPair(DKConstant.DK_CM_PARM_END, null);
	  dkResultSetCursor cursor = dkDatastore.execute(qs, DKConstantICM.DK_CM_XQPE_QL_TYPE, options);
//	  assert cursor.cardinality() == 1;
	  DKDDO ddo = cursor.fetchNext();
	  DKPidICM pid = (DKPidICM) ddo.getPidObject();
	  pidString = pid.toString();
	  cursor.destroy();
	}
	catch (Exception exc)
	{
	  throw new IntegrationException(exc);
	}
	finally
	{
	  // Ensure that connection is returned to pool regardless of how we return
	  if (dkDatastore != null)
	  {
		try
		{
		  ManageConnection.disconnect(dkDatastore);
		}
		catch (Exception e)
		{
		}
	  }
	}
	return pidString;
  }

  /**
   * Get a list of the item type and attribute names in Content Manager.
   * These names can be used in a UI to select corresponding attributes
   * @return A map where the item type name is the key and a list
   * of the attribute names is in the value's field.
   */
  public Map getItemTypeAttributeNames()
  {
    TxHandle globalTransaction = null;
    DKDatastoreICM dkDatastore = null;
    HashMap itemTypeAttributeNames = new HashMap();

    try
    {
      // Suspend the current global transaction to permit use of CM API in case of connection pooling

      globalTransaction = suspendGlobalTransaction();
      dkDatastore = ManageConnection.connect();
      // Get the entities in the data store
      DKSequentialCollection pCol = (DKSequentialCollection) dkDatastore.listEntities();
      dkIterator pIter = pCol.createIterator();
      while (pIter.more() == true)
      {
        DKComponentTypeDefICM itemTypeDef = (DKComponentTypeDefICM) pIter.next();
        String itemTypeName = itemTypeDef.getName();
        // Get the attribute to the item type
        DKSequentialCollection pCol2 =
          (DKSequentialCollection) dkDatastore.listEntityAttrs(itemTypeName);
        dkIterator pIter2 = pCol2.createIterator();
        List attributeNames = new ArrayList();
        while (pIter2.more() == true)
        {
          DKAttrDefICM attrDef = (DKAttrDefICM) pIter2.next();
          attributeNames.add(attrDef.getName());
        }
        // Add itemType and its attributes to the map
        itemTypeAttributeNames.put(itemTypeName, attributeNames);
      }
    }
    catch (Exception e)
    {
      throw IntegrationException.wrap(e);
    }
    finally
    {
      if (dkDatastore != null)
      {
        try
        {
          ManageConnection.disconnect(dkDatastore);
        }
        catch (Exception ignored)
        {
        }
      }
      if (globalTransaction != null)
      {
        resumeGlobalTransaction(globalTransaction);
      }
    }
    return itemTypeAttributeNames;
  }

  /**
  	* Get an attribute value of items in the specified folder (e.g. the license plate attribute
  	* in the adjuster's report item type).
  	* @param folderPidString - PID of folder associated with BPC process
  	* @param attributeSpecification - Attributes to retrieve in &lt;item-type&gt;/&lt;attribute-name&gt; 
  	* format, e.g. <code>"XYZ_ClaimForm/XYZ_DriversLic"</code>.
  	* Note that &lt;attribute-name&gt; can include child component names,
  	* e.g. Book/Chapter/Section/Author.
  	* @return an Object for the attribute value or null if the attribute cannot be resolved.		
  	*/
  public Object getFolderContentAttribute(String folderPidString, String attributeSpecification)
  {
    Map map = getFolderContentAttributes(folderPidString, new String[] { attributeSpecification });
    return map.get(attributeSpecification);
  }

  /**
  	* Get an attribute value of the associated item (when it is no folder).
  	* @param itemPidString - PID of item containing attributes.
  	* @param attributeSpecification - Attribute to retrieve in &lt;item-type&gt;/&lt;attribute-name&gt; 
  	* format, e.g. <code>"XYZ_ClaimForm/XYZ_DriversLic"</code>.
  	* Note that &lt;attribute-name&gt; can include child component names,
  	* e.g. Book/Chapter/Section/Author.
  	* @return an Object for the attribute value or null if the attribute cannot be resolved.
  	*/
  public Object getContentAttribute(String itemPidString, String attributeSpecification)
  {
    Map map = getContentAttributes(itemPidString, new String[] { attributeSpecification });
    return map.get(attributeSpecification);
  }

  /**
   * Get the attributes of items in the specified folder (e.g. the license plate attribute
   * in the adjuster's report item type that is located in an auto claims folder).
   * @param folderPidString - PID of folder that is routed through a process
   * @param attributeSpecifications - Attributes to retrieve in &lt;item-type&gt;/&lt;attribute-name&gt; 
   * format, e.g.
   * <code><pre>
   * new String[]
   * {
   *     "XYZ_ClaimForm/XYZ_DriversLic",
   *     "XYZ_ClaimForm/XYZ_PolicyNum",
   *     "XYZ_AdjReport/XYZ_LicPlate"
   * }
   * </pre></code>
   * Note that &lt;attribute-name&gt; can include child component names,
   * e.g. Book/Chapter/Section/Author.
   * @return a map that maps attribute specs to the corresponding values
   */
  public Map getFolderContentAttributes(
    String folderPidString,
    String[] attributeSpecifications)
  {
    Map map = null;
    try
    {
      String itemId = new DKPidICM(folderPidString).getItemId();
      String query =
        "/*[@ITEMID=\"" + itemId + "\"]/OUTBOUNDLINK[@LINKTYPE=\"DKFolder\"]/@TARGETITEMREF => *[@VERSIONID = latest-version(.)]";
      map = retrieveAttributes(query, attributeSpecifications);
    }
    catch (Exception e)
    {
      throw IntegrationException.wrap(e);
    }
    return map;
  }

  /**
   * Get the attributes of of the specified item.
   * @param itemPidString - PID of item containing attributes.
   * @param attributeSpecifications - Attributes to retrieve in &lt;item-type&gt;/&lt;attribute-name&gt; format, e.g.
   * <code><pre>
   * new String[]
   * {
   *     "XYZ_ClaimForm/XYZ_DriversLic",
   *     "XYZ_ClaimForm/XYZ_PolicyNum",
   *     "XYZ_AdjReport/XYZ_LicPlate"
   * }
   * </pre></code>
   * Note that &lt;attribute-name&gt; can include child component names,
   * e.g. Book/Chapter/Section/Author.
    * @return a map that maps attribute specs to the corresponding values
  */
  public Map getContentAttributes(String itemPidString, String[] attributeSpecifications)
  {
    Map map = null;
    try
    {
      String itemId = new DKPidICM(itemPidString).getItemId();
      String query = "/*[@ITEMID=\"" + itemId + "\"]";
      map = retrieveAttributes(query, attributeSpecifications);
    }
    catch (Exception e)
    {
      throw IntegrationException.wrap(e);
    }
    return map;
  }
  
  /**
   * Retrieve the attribute values specified by the query and an attributeSpecification filter.
   * Return a map that maps attribute specs to the corresponding attribute values.
   * Note that this restricts the use of the getFolderContentAttributes operations to attribute specs
   * that are unique within a folder. If attribute lookup needs to be done on e.g. multiple instances
   * of an item type, this method needs to be modified to return either a list of values, or a value
   * that is derived from the different values e.g. based on a formula (minimum, maximum or arithmetic mean...). 
   * 
   * @param query an XPath query that specifies the items to retrieve
   * @param attributeSpecifications an array of attribute specifications in the form <itemName>/<attrName>
   * @return a map that aligns attribute specs with the corresponding values (null if they do not exist)
   */

  public Map retrieveAttributes(String query, String[] attributeSpecifications)
  {
    Map values = null;
    TxHandle globalTransaction = null;
    DKDatastoreICM dkDatastore = null;

    try
    {
      // Suspend the current global transaction to permit use of CM API in case of connection pooling
      globalTransaction = suspendGlobalTransaction();
      dkDatastore = ManageConnection.connect();

      // Get the (potentially cached) query options for the attribute specs
      QueryOptions queryOptions = QueryOptions.getQueryOptions(attributeSpecifications);

      DKNVPair options[] = queryOptions.getOptions();
      Map itemTypes = queryOptions.getItemTypes();

      // Execute the query
      dkResultSetCursor results =
        dkDatastore.execute(query, DKConstant.DK_CM_XQPE_QL_TYPE, options);

      DKDDO ddo = null;
      values = new HashMap();

      // For each item instance returned by the query
      while ((ddo = results.fetchNext()) != null)
      {
        // Get the persistent identifier for this item instance
        DKPidICM itemPid = (DKPidICM) ddo.getPidObject();

        // Get the item type name for this item instance
        String itemTypeName = itemPid.getObjectType();

        // The query results may contain system data (e.g. ICMParts<nnnn>)
        DKSequentialCollection attributeNames =
          (DKSequentialCollection) itemTypes.get(itemTypeName);

        // The projection list appears to be a filtering specification,
        // not a retrieval specification. All attributes are returned for item
        // types that match the query, but don't match the projection list.

        //Assert.assertion(attributeNames != null, "Unrequested item type");
        if (attributeNames != null)
        {
          // So we extract what we want rather than saving everything
          dkIterator attributeNameIterator = attributeNames.createIterator();

          // While there are requested attributes for this item type        
          while (attributeNameIterator.more())
          {
            // Get the attribute name
            String attributeName = (String) attributeNameIterator.next();

            // Get the attribute's value
            Object attributeValue = ddo.getDataByName(attributeName);

            // Store the attribute value in the result map
            // The value may be null if it was not in the result set.
            values.put(itemTypeName + '/' + attributeName, attributeValue);
          }
        }
      }
    }
    catch (Exception e)
    {
      throw IntegrationException.wrap(e);
    }
    finally
    {
      if (dkDatastore != null)
      {
        try
        {
          ManageConnection.disconnect(dkDatastore);
        }
        catch (Exception ignored)
        {
        }
      }
      if (globalTransaction != null)
      {
        resumeGlobalTransaction(globalTransaction);
      }
    }
    return values;
  }

  /**
   * Evaluate a Content Manager XPath expression.
   * Use this function in a Process Choreographer transition condition to
   * determine a flow route based on document contents.
   * @param xPath A Content Manager XPath expression for a condition
   * @return true, if the condition is true
   */
  public boolean evaluateCondition(String xPath)
  {
    boolean isConditionTrue = false;
    DKDatastoreICM dkDatastore = null;
    TxHandle globalTransaction = null;
    try
    {
      globalTransaction = suspendGlobalTransaction();
      dkDatastore = ManageConnection.connect();
      DKNVPair options[] =
        new DKNVPair[] {
          new DKNVPair(DKConstant.DK_CM_PARM_MAX_RESULTS, "1"),
          new DKNVPair(
            DKConstant.DK_CM_PARM_RETRIEVE,
            new Integer(DKConstant.DK_CM_CONTENT_IDONLY)),
          new DKNVPair(DKConstant.DK_CM_PARM_END, null)};
      DKResults results =
        (DKResults) dkDatastore.evaluate(xPath, DKConstantICM.DK_CM_XQPE_QL_TYPE, options);
      isConditionTrue = results.cardinality() > 0;
    }
    catch (Exception e)
    {
      throw IntegrationException.wrap(e);
    }
    finally
    {
      if (dkDatastore != null)
      {
        try
        {
          ManageConnection.disconnect(dkDatastore);
        }
        catch (Exception ignored)
        {
        }
      }
      if (globalTransaction != null)
      {
        resumeGlobalTransaction(globalTransaction);
      }
    }
    return isConditionTrue;
  }

  /**
   * Add the results of a Content Manager query to the folder that is routed through the process.
   * 
   * @param folderPidString PID string of the folder that is routed through the process
   * @param query XPath query string that specifies the items to be added
   * @return number of items that have been added to the folder (number of hits)
   */

  public int addItemToFlowFolder(String folderPidString, String query)
  {
    DKDatastoreICM dkDatastore = null;
    DKDDO          ddoFolder   = null;
    TxHandle globalTransaction = null;
    int numberOfHits = 0;
    
    try
    {
      // Suspend the current global transaction to permit use of CM API in case of connection pooling
      globalTransaction = suspendGlobalTransaction();
      dkDatastore = ManageConnection.connect();
      DKNVPair options[] = new DKNVPair[2];
      options[0] =
        new DKNVPair(DKConstant.DK_CM_PARM_RETRIEVE, new Integer(DKConstant.DK_CM_CONTENT_IDONLY));
      options[1] = new DKNVPair(DKConstant.DK_CM_PARM_END, null);
      dkResultSetCursor cursor =
        dkDatastore.execute(query, DKConstantICM.DK_CM_XQPE_QL_TYPE, options);
      numberOfHits = cursor.cardinality();
      if (numberOfHits > 0)
      {
        ddoFolder = dkDatastore.createDDO(folderPidString);
        ddoFolder.retrieve(DKConstant.DK_CM_CONTENT_LINKS_OUTBOUND);
        // Outbound Links must be specified.

        short dataid = ddoFolder.dataId(DKConstant.DK_CM_NAMESPACE_ATTR, DKConstant.DK_CM_DKFOLDER);

        if (dataid == 0)
          throw new IntegrationException("Flow item must be a folder for 'addItemToFlowFolder' to apply.");

        DKFolder dkFolder = (DKFolder) ddoFolder.getData(dataid);
        DKDDO ddo;
        try
        {
          while ((ddo = cursor.fetchNext()) != null)
            dkFolder.addElement(ddo);
            dkDatastore.checkOut(ddoFolder);
            ddoFolder.update();
            dkDatastore.checkIn(ddoFolder);
        }
        catch (DKUsageError ue)
        {
          System.err.println("!> addItemToFLowFolder: " + ue.getMessage());
        }
      }
      cursor.destroy();
    }
    catch (Exception e)
    {
      throw IntegrationException.wrap(e);
    }
    finally {
      if (dkDatastore != null) {
      	if (ddoFolder != null) {
          	try {
        		dkDatastore.checkIn(ddoFolder); // try to check it in
            }
            catch (Exception exc) {} // it is probably already checked in - nothing to do
        }
        try {
        	ManageConnection.disconnect(dkDatastore);
        }
        catch (Exception ignored) {}
      }
      if (globalTransaction != null) resumeGlobalTransaction(globalTransaction);
    }
    return numberOfHits;
  }

  /**
   * Add the item routed through the process to the folder retrieved by a Content Manager query. 
   * 
   * @param pidString PID string of the item that is routed through the process
   * @param query XPath query string that specifies the folder to which this item should be added
   * @return number of folders that matched the query
   */

  public int addFlowItemToFolder(String pidString, String query)
  {
    DKDatastoreICM dkDatastore = null;
    DKDDO          ddoFolder   = null;
    TxHandle globalTransaction = null;
    int numberOfHits = 0;
    try
    {
      // Suspend the current global transaction to permit use of CM API in case of connection pooling
      globalTransaction = suspendGlobalTransaction();
      dkDatastore = ManageConnection.connect();
      DKNVPair options[] = new DKNVPair[2];
      options[0] =
        new DKNVPair(
          DKConstant.DK_CM_PARM_RETRIEVE,
          new Integer(DKConstant.DK_CM_CONTENT_LINKS_OUTBOUND));
      options[1] = new DKNVPair(DKConstant.DK_CM_PARM_END, null);
      dkResultSetCursor cursor =
        dkDatastore.execute(query, DKConstantICM.DK_CM_XQPE_QL_TYPE, options);
      numberOfHits = cursor.cardinality();
      if (numberOfHits > 0)
      {
        ddoFolder = cursor.fetchNext();
        short dataid = ddoFolder.dataId(DKConstant.DK_CM_NAMESPACE_ATTR, DKConstant.DK_CM_DKFOLDER);

        if (dataid == 0)
          throw new IntegrationException("Retrieved item must be a folder for 'addFlowItemToFolder' to apply.");

        DKFolder dkFolder = (DKFolder) ddoFolder.getData(dataid);
        DKDDO ddo = dkDatastore.createDDO(pidString);
        try
        {
          dkFolder.addElement(ddo);
          dkDatastore.createDDO(pidString);
          dkDatastore.checkOut(ddoFolder);
          ddoFolder.update();
          dkDatastore.checkIn(ddoFolder);
        }
        catch (DKUsageError ue)
        {
          System.err.println("!> addFlowItemToFolder: " + ue.getMessage());
        }
      }
      cursor.destroy();
    }
    catch (Exception e)
    {
      throw IntegrationException.wrap(e);
    }
    finally {
        if (dkDatastore != null) {
        	if (ddoFolder != null) {
            	try {
            		dkDatastore.checkIn(ddoFolder); // try to check it in
            	}
            	catch (Exception ignored) {} // it is probably already checked in - nothing to do    		
        	}
        	try {
        		ManageConnection.disconnect(dkDatastore);
        	}
        	catch (Exception ignored) {}
        }
        if (globalTransaction != null) resumeGlobalTransaction(globalTransaction);
    }
    return numberOfHits;
  }

  /**
   * Assign the item routed through the process to a new item type.
   * Note: this changes the itemPid so it needs to be reassigned to the process
   * property when reindex() has been called to ensure the client displays
   * the appropriate set of attributes/parts.
   * 
   * @param pidString PID string of the item that is routed through the process
   * @param newItemType name of the new item type the item should be assigned to
   * @return number of folders that matched the query
   */

  public void reindex(String pidString, String newItemType)
  {
    DKDatastoreICM dkDatastore = null;
    TxHandle globalTransaction = null;
    DKDDO ddo = null;
  	   
    try
    {
      // Suspend the current global transaction to permit use of CM API in case of connection pooling
      globalTransaction = suspendGlobalTransaction();
      dkDatastore = ManageConnection.connect();
      DKDatastoreDefICM dsDef = new DKDatastoreDefICM(dkDatastore);
      
      ddo = dkDatastore.createDDO(pidString);

      dkDatastore.checkOut(ddo);
      ddo.retrieve();
      dkDatastore.moveObject(ddo, newItemType);
      ddo.update();
      dkDatastore.checkIn(ddo);
    }
    catch (Exception e)
    {
      throw IntegrationException.wrap(e);
    }
    finally
    {
      try {
      		if (ddo != null) dkDatastore.checkIn(ddo); // try to check it in
      }
      	catch (Exception ignored) {} // it is probably already checked in - nothing to do
      if (dkDatastore != null)
      {
        try
        {
          if (ddo != null) dkDatastore.checkIn(ddo);
          ManageConnection.disconnect(dkDatastore);
        }
        catch (Exception ignored)
        {
        }
      }
      if (globalTransaction != null)
      {
        resumeGlobalTransaction(globalTransaction);
      }
    }
  }
 
  
  /**
   * Set the ACL of an item to a new value.
   * @param dsDef datastore definition. Needed to validate if item-level ACL checking is enabled.
   * @param ddo ddo corresponding to the item to be updated
   * @param acl new ACL
   * @throws Exception problem retrieving the item or updating its ACL property
   */
  
  private void setItemACL(DKDatastoreDefICM dsDef,
  						  DKDDO ddo,
  						  String acl) throws Exception
  {
    ddo.retrieve(DKConstant.DK_CM_CONTENT_LINKS_OUTBOUND);

    String entity = ddo.getPidObject().getObjectType();
    DKItemTypeDefICM itDef = (DKItemTypeDefICM)dsDef.retrieveEntity(entity);
    if (itDef.getItemLevelACLFlag() == 0)
      throw new IntegrationException("The setACL service needs item level ACL checking to be enabled.");

    // Outbound Links must be specified.
    short propId = ddo.propertyId(DKConstantICM.DK_ICM_PROPERTY_ACL);
    if (propId == 0) propId = ddo.addProperty(DKConstantICM.DK_ICM_PROPERTY_ACL);
    ddo.setProperty(propId, acl);
  }

  /**
   * Set the ACL of the item routed through the process to a new value. If this item is a folder,
   * recursively change set the ACLs of all items contained in this folder.
   * 
   * @param pidString PID string of the item that is routed through the process
   * @param acl name of the new ACL to be assigned to the item and its contents
   * @return number of folders that matched the query
   */

  public void setACL(String pidString, String acl)
  {
    DKDatastoreICM dkDatastore = null;
    TxHandle globalTransaction = null;
    DKDDO ddo = null;
    DKDDO ddoInFolder = null;
  	   
    try
    {
      // Suspend the current global transaction to permit use of CM API in case of connection pooling
      globalTransaction = suspendGlobalTransaction();
      dkDatastore = ManageConnection.connect();
      DKDatastoreDefICM dsDef = new DKDatastoreDefICM(dkDatastore);
      
      ddo = dkDatastore.createDDO(pidString);

      dkDatastore.checkOut(ddo);
      setItemACL(dsDef, ddo, acl);
      
      short dataid = ddo.dataId(DKConstant.DK_CM_NAMESPACE_ATTR, DKConstant.DK_CM_DKFOLDER);

      // if the process item is a folder: apply ACL change recursively to the folder's contents
      // note: this implementation of the service supports only one level of embedding
      if (dataid != 0) {
      	DKFolder dkFolder = (DKFolder) ddo.getData(dataid);
        dkIterator iter = dkFolder.createIterator();
        while(iter.more()){    
            ddoInFolder = (DKDDO) iter.next();
            dkDatastore.checkOut(ddoInFolder);
            // ensure a single ACL update failure does not stop the update from proceeding
            try {
            	setItemACL(dsDef, ddoInFolder, acl);
            }
            catch (Exception e) {
                throw IntegrationException.wrap(e);            	
            }
            ddoInFolder.update();
            dkDatastore.checkIn(ddoInFolder);
        }
      }
      ddo.update();
      dkDatastore.checkIn(ddo);
    }
    catch (Exception e)
    {
      throw IntegrationException.wrap(e);
    }
    finally
    {
      if (dkDatastore != null)
      {
      	try {
      		if (ddo != null) dkDatastore.checkIn(ddo); // try to check it in
      	}
      	catch (Exception ignored) {} // it is probably already checked in - nothing to do
      	try {
      		if (ddoInFolder != null) dkDatastore.checkIn(ddoInFolder); // try to check it in
      	}
      	catch (Exception ignored) {} // it is probably already checked in - nothing to do
        try {
          if (ddoInFolder != null) dkDatastore.checkIn(ddoInFolder);
          ManageConnection.disconnect(dkDatastore);
        }
        catch (Exception ignored)
        {
        }
      }
      if (globalTransaction != null)
      {
        resumeGlobalTransaction(globalTransaction);
      }
    }
  }

  // Some variables for allowing local transactions
  ExtensionHelper extHelper = null;
  TransactionControl tranControl = null;
}
