/*
@copyright(disclaimer)

Licensed Materials - Property of IBM
IBM DB2 Content Manager Enterprise Edition V8  (program number 5724-B19)
IBM DB2 Content Manager Express Edition V8 (program number 5724-F73)
IBM DB2 Content Manager for z/OS V8 (program number 5697-H60)
(c ) Copyright IBM Corp. 2005. All Rights Reserved.

US Government Users Restricted Rights
Use, duplication or disclosure restricted by GSA ADP Schedule
Contract with IBM Corp.

DISCLAIMER OF WARRANTIES :

Permission is granted to copy and modify this  Sample code, and to
distribute modified versions provided that both the copyright
notice,- and this permission notice and warranty disclaimer  appear in all copies and modified
versions.

THIS SAMPLE CODE IS LICENSED TO YOU AS-IS.
IBM  AND ITS SUPPLIERS AND LICENSORS  DISCLAIM
ALL WARRANTIES, EITHER EXPRESS OR IMPLIED, IN SUCH SAMPLE CODE, INCLUDING THE WARRANTY
OF NON-INFRINGEMENT AND THE IMPLIED WARRANTIES OF MERCHANTABILITY OR FITNESS FOR A
PARTICULAR PURPOSE. IN NO EVENT WILL IBM OR ITS LICENSORS OR SUPPLIERS BE LIABLE FOR ANY
DAMAGES ARISING OUT OF THE USE OF  OR INABILITY TO USE THE SAMPLE CODE, DISTRIBUTION OF
THE SAMPLE CODE, OR COMBINATION OF THE SAMPLE CODE WITH ANY OTHER CODE. IN NO EVENT SHALL
IBM OR ITS LICENSORS AND SUPPLIERS BE LIABLE FOR ANY LOST REVENUE, LOST PROFITS OR DATA,
OR FOR DIRECT, INDIRECT, SPECIAL, CONSEQUENTIAL,INCIDENTAL OR PUNITIVE DAMAGES, HOWEVER
CAUSED AND REGARDLESS OF THE THEORY OF LIABILITY,-, EVEN IF IBM OR ITS LICENSORS OR SUPPLIERS
HAVE BEEN ADVISED OF THE POSSIBILITY OF SUCH DAMAGES.

@endCopyright
*/

package com.ibm.bpe.cm.util;

import com.ibm.bpe.api.QueryResultSet;

/**
 * Map column positions in a query result set taking care of a potential offset
 * due to columns that should not be considered (columnsToSkip).
 */

public class QueryResultSetAdapter
{
  private QueryResultSet _queryResultSet;
  private int _columnsToSkip;

  /**
   * Initialize the QueryResultSetAdapter based on a query result set and offset
   * @param queryResultSet the result set at issue
   * @param columnsToSkip offset
   */
  
  public QueryResultSetAdapter(QueryResultSet queryResultSet, int columnsToSkip)
  {
    _queryResultSet = queryResultSet;
    _columnsToSkip = columnsToSkip;
  }
  
  /**
   * Return next column
   * @return next columh
   */
  
  public boolean next()
  {
    return _queryResultSet.next();
  }
  
  /**
   * Return column at adjusted index
   * @param columnIndex 
   * @return
   */
  
  public String getString(int columnIndex)
  {
    return _queryResultSet.getString(columnIndex + _columnsToSkip);
  }
  
  /**
   * Return column at index without adjustment
   * @param columnIndex
   * @return
   */
  
  public String getAbsoluteString(int columnIndex)
  {
    return _queryResultSet.getString(columnIndex);
  }
  
  /**
   * Return initial column
   * @return
   */
  
  public boolean first()
  {
    return _queryResultSet.first();
  }
  
  /**
   * Return adjusted number of columns
   * @return
   */
  
  public int numberColumns()
  {
    return _queryResultSet.numberColumns() - _columnsToSkip;
  }
  
  /**
   * Return display name of column at adjusted index position
   * @param columnIndex
   * @return
   */
  
  public String getColumnDisplayName(int columnIndex)
  {
    return _queryResultSet.getColumnDisplayName(columnIndex + _columnsToSkip);
  }
  
  /**
   * Return type name of column at adjusted index position
   * @param columnIndex
   * @return
   */
  
  public short getColumnType(int columnIndex)
  {
    return _queryResultSet.getColumnType(columnIndex + _columnsToSkip);
  }
  
  /**
   * Return object ID at adusted index position
   * @param columnIndex
   * @return
   */
  
  public Object getOID(int columnIndex)
  {
    return _queryResultSet.getOID(columnIndex + _columnsToSkip);
  }
  
  /**
   * Return object ad adjusted index position
   * @param columnIndex
   * @return
   */
  
  public Object getObject(int columnIndex)
  {
    return _queryResultSet.getObject(columnIndex + _columnsToSkip);
  }
}