/*
@copyright(disclaimer)

Licensed Materials - Property of IBM
IBM DB2 Content Manager Enterprise Edition V8  (program number 5724-B19)
IBM DB2 Content Manager Express Edition V8 (program number 5724-F73)
IBM DB2 Content Manager for z/OS V8 (program number 5697-H60)
(c ) Copyright IBM Corp. 2005. All Rights Reserved.

US Government Users Restricted Rights
Use, duplication or disclosure restricted by GSA ADP Schedule
Contract with IBM Corp.

DISCLAIMER OF WARRANTIES :

Permission is granted to copy and modify this  Sample code, and to
distribute modified versions provided that both the copyright
notice,- and this permission notice and warranty disclaimer  appear in all copies and modified
versions.

THIS SAMPLE CODE IS LICENSED TO YOU AS-IS.
IBM  AND ITS SUPPLIERS AND LICENSORS  DISCLAIM
ALL WARRANTIES, EITHER EXPRESS OR IMPLIED, IN SUCH SAMPLE CODE, INCLUDING THE WARRANTY
OF NON-INFRINGEMENT AND THE IMPLIED WARRANTIES OF MERCHANTABILITY OR FITNESS FOR A
PARTICULAR PURPOSE. IN NO EVENT WILL IBM OR ITS LICENSORS OR SUPPLIERS BE LIABLE FOR ANY
DAMAGES ARISING OUT OF THE USE OF  OR INABILITY TO USE THE SAMPLE CODE, DISTRIBUTION OF
THE SAMPLE CODE, OR COMBINATION OF THE SAMPLE CODE WITH ANY OTHER CODE. IN NO EVENT SHALL
IBM OR ITS LICENSORS AND SUPPLIERS BE LIABLE FOR ANY LOST REVENUE, LOST PROFITS OR DATA,
OR FOR DIRECT, INDIRECT, SPECIAL, CONSEQUENTIAL,INCIDENTAL OR PUNITIVE DAMAGES, HOWEVER
CAUSED AND REGARDLESS OF THE THEORY OF LIABILITY,-, EVEN IF IBM OR ITS LICENSORS OR SUPPLIERS
HAVE BEEN ADVISED OF THE POSSIBILITY OF SUCH DAMAGES.

@endCopyright
*/


package com.ibm.bpe.cm.ejb;

import java.util.List;

import javax.naming.InitialContext;
import javax.naming.NamingException;

import com.ibm.bpe.api.LocalBusinessFlowManagerHome;
import com.ibm.bpe.cm.util.CombinedQuery;
import com.ibm.bpe.cm.util.IntegrationException;

/**
 * Bean implementation class for Enterprise Bean: CMBPCIntegration
 */
public class CMBPCIntegrationBean implements javax.ejb.SessionBean {
	private javax.ejb.SessionContext mySessionCtx;
	/**
	 * getSessionContext
	 */
	public javax.ejb.SessionContext getSessionContext() {
		return mySessionCtx;
	}
	/**
	 * setSessionContext
	 */
	public void setSessionContext(javax.ejb.SessionContext ctx) {
		mySessionCtx = ctx;
	}
	/**
	 * ejbCreate
	 */
	public void ejbCreate() throws javax.ejb.CreateException {
		
		try {
	      InitialContext initialContext = new InitialContext();
	      //
	      // Get the home interface for the WPC generic API
	      //					
		  LocalBusinessFlowManagerHome lbfmHome =
				 (LocalBusinessFlowManagerHome) initialContext.lookup("local:ejb/com/ibm/bpe/api/BusinessFlowManagerHome");				
	      //
	      // Create the object implementing the combined query
	      //
	      _combinedQuery = new CombinedQuery(lbfmHome, null);
	    }
	    catch (NamingException exc)
	    {
	      exc.printStackTrace();
	    }
	}
	/**
	 * ejbActivate
	 */
	public void ejbActivate() {
	}
	/**
	 * ejbPassivate
	 */
	public void ejbPassivate() {
	}
	/**
	 * ejbRemove
	 */
	public void ejbRemove() {
	}
	
	/**
	 * Run a combined query for Process Choreographer and Content Manager data.
	 * For more details on the QueryResultSet and how to specify the clauses, also refer
	 * to the Process Choreographer documentation.<br>
	 * <b>Note:</b> This function is also available on the <code>ContentManager</code> business
	 * deletage class. Use local EJB calls and this function to get better performance when 
	 * calling <code>combinedQuery</code> from the context of an EJB or a Servlet.<br>  
     * @param selectClauseWorkflow The requested Process Choreographer column names. Must not be null.
     * @param flowItemTypeNameContent The item type of the item which is routed through the flow
     * This can be a document item type or a folder item type. Must not be null.
     * @param flowItemTypeAttributesContent A comma separated list of attributes of the flow item type
     * that should be returned in the result set. 
     * @param additionalAttributesContent If the flowItemTypeContent is a folder, this specifies an 
     * optional list of attributes of the contained documents. The list is comma separated and
     * the attributes are specified in the form <code>&lt;itemtype&gt;/&lt;attribute&gt;</code>.   
     * @param whereClauseWorkflow The SQL where condition for Process Choreographer data, may be null.
     * @param xPathContent  An valid XPath expression to filter the Content Manager data. The result must
     * be the flow item type. To express conditions on contained item attributes make sure
     * to use an inboundlink to get to the folder item. 	  	
     * @param orderByClauseWorkflow The order clause for Process Choreographer data, may be null.
     * @param timezoneWorkflow A timezone specification for timestamp given or returned, may be null. 
     * @return The result of the combined query. Null, if an integration exception has occured.
    */
  
	public List combinedQuery(
			String selectClauseWorkflow,
			String flowItemTypeNameContent,
			String flowItemTypeAttributesContent,
			String additionalAttributesContent,
			String whereClauseWorkflow,
			String xPathContent,
			String orderByClauseWorkflow,    
			java.util.TimeZone timezoneWorkflow)
  {
    List result = null;
    
    try {
    	result = _combinedQuery.combinedQuery(
    		      selectClauseWorkflow,
			      flowItemTypeNameContent,
			      flowItemTypeAttributesContent,
			      additionalAttributesContent,
			      whereClauseWorkflow,
			      xPathContent,
			      orderByClauseWorkflow,      
			      timezoneWorkflow).getList();
    }
    catch (IntegrationException e) {
    	e.printStackTrace();
    }
    return result;
  }
  

	
	CombinedQuery _combinedQuery = null;
}
