/*
@copyright(disclaimer)

Licensed Materials - Property of IBM
IBM DB2 Content Manager Enterprise Edition V8  (program number 5724-B19)
IBM DB2 Content Manager Express Edition V8 (program number 5724-F73)
IBM DB2 Content Manager for z/OS V8 (program number 5697-H60)
(c ) Copyright IBM Corp. 2005. All Rights Reserved.

US Government Users Restricted Rights
Use, duplication or disclosure restricted by GSA ADP Schedule
Contract with IBM Corp.

DISCLAIMER OF WARRANTIES :

Permission is granted to copy and modify this  Sample code, and to
distribute modified versions provided that both the copyright
notice,- and this permission notice and warranty disclaimer  appear in all copies and modified
versions.

THIS SAMPLE CODE IS LICENSED TO YOU AS-IS.
IBM  AND ITS SUPPLIERS AND LICENSORS  DISCLAIM
ALL WARRANTIES, EITHER EXPRESS OR IMPLIED, IN SUCH SAMPLE CODE, INCLUDING THE WARRANTY
OF NON-INFRINGEMENT AND THE IMPLIED WARRANTIES OF MERCHANTABILITY OR FITNESS FOR A
PARTICULAR PURPOSE. IN NO EVENT WILL IBM OR ITS LICENSORS OR SUPPLIERS BE LIABLE FOR ANY
DAMAGES ARISING OUT OF THE USE OF  OR INABILITY TO USE THE SAMPLE CODE, DISTRIBUTION OF
THE SAMPLE CODE, OR COMBINATION OF THE SAMPLE CODE WITH ANY OTHER CODE. IN NO EVENT SHALL
IBM OR ITS LICENSORS AND SUPPLIERS BE LIABLE FOR ANY LOST REVENUE, LOST PROFITS OR DATA,
OR FOR DIRECT, INDIRECT, SPECIAL, CONSEQUENTIAL,INCIDENTAL OR PUNITIVE DAMAGES, HOWEVER
CAUSED AND REGARDLESS OF THE THEORY OF LIABILITY,-, EVEN IF IBM OR ITS LICENSORS OR SUPPLIERS
HAVE BEEN ADVISED OF THE POSSIBILITY OF SUCH DAMAGES.

@endCopyright
*/

//
// 5639-D57, 5630-A36, 5630-A37, 5724-D18 
//(C) COPYRIGHT International Business Machines Corp. 1997, 2003
// All Rights Reserved * Licensed Materials - Property of IBM
//
// ----------------------------------------------------------------------
// This program may be used, executed, copied, modified and distributed 
// without royalty for the purpose of developing, using, marketing, or 
// distributing.
// ----------------------------------------------------------------------
//
// This file implements a custom user registry based on the information
// found in the WebSphere Application Server information center:
// http://publib.boulder.ibm.com/infocenter/wasinfo/v5r1//index.jsp?topic=/com.ibm.wasee.doc/info/ee/ae/rsec_frsjf502.html
//

package com.ibm.bpe.cm.conmgmt;

import java.rmi.RemoteException;
import java.security.cert.X509Certificate;
import java.util.ArrayList;
import java.util.List;
import java.util.Vector;

import com.ibm.mm.sdk.common.DKDatastoreAdminICM;
import com.ibm.mm.sdk.common.dkIterator;
import com.ibm.mm.sdk.common.dkUserDef;
import com.ibm.mm.sdk.common.dkUserGroupDef;
import com.ibm.mm.sdk.common.dkUserManagement;
import com.ibm.mm.sdk.server.DKDatastoreICM;
import com.ibm.websphere.security.CertificateMapFailedException;
import com.ibm.websphere.security.CertificateMapNotSupportedException;
import com.ibm.websphere.security.EntryNotFoundException;
import com.ibm.websphere.security.NotImplementedException;
import com.ibm.websphere.security.PasswordCheckFailedException;
import com.ibm.websphere.security.Result;
import com.ibm.websphere.security.UserRegistry;

/**
 * A UserRegistry implementation to access the Content Manager
 * users and group definitions.
 * This code requires access to the CM libaray server.<br>
 * <b>Note:</b>
 * Make sure the following is set in the Application Server
 * <ul>
 * <li>The classpath points to the CM jars</li>
 * <li>db2java.zip is on the classpath </li>
 * <li>The Security->UserRegistries->Custom entry points to this class</li>
 * <li>Custom properties for this registry are set (names/values see consant Strings below)</li>
 * <li>Security->GlobalSecurity ActiveUserRegistry is set to "Custom"
 * <li>In Security->GlobalSecurity, Java 2 security is turned off (current limitation)</li> 
 * </ul>
 * When running in UTE, edit security.xml as follows (remember to save again during server startup since
 * this file is overwritten after the server prepared to start, well ... UTE isn't really designed to run
 * with a different user registry).
 * Modify line in security.xml:
 * <security:Security xmi:version="2.0" xmlns:xmi="http://www.omg.org/XMI" xmlns:orb.securityprotocol="http://www.ibm.com/websphere/appserver/schemas/5.0/orb.securityprotocol.xmi" xmlns:security="http://www.ibm.com/websphere/appserver/schemas/5.0/security.xmi" xmi:id="Security_1" useLocalSecurityServer="true" useDomainQualifiedUserNames="false" enabled="true" cacheTimeout="600" issuePermissionWarning="true" activeProtocol="BOTH" enforceJava2Security="false" activeAuthMechanism="SWAMAuthentication_1" activeUserRegistry="CustomUserRegistry_1" defaultSSLSettings="SSLConfig_1">
 * Modify custom user registry definition in security.xml:
 * <userRegistries xmi:type="security:CustomUserRegistry" xmi:id="CustomUserRegistry_1" serverId="icmadmin" serverPassword="{xor}LD4raWgsPis=" realm="ContentManager" limit="0" ignoreCase="false" customRegistryClassName="com.ibm.bpe.cm.staff.ContentManagerWebSphereUserRegistry"/>
 * <br/>
 * <b>Note:</b>
 * This code assumes that "userSecurityName" and "unique user Id" are the same for 
 * this content manager based implementation. This simplifies the code and mapping between both.
 **/
public class ContentManagerWebSphereUserRegistry implements UserRegistry
{

	/** Default Constructor **/
	public ContentManagerWebSphereUserRegistry()
		throws java.rmi.RemoteException
	{
	}

	/**
	 * Initializes the registry. This method is called when creating the
	 * registry.
	 * A property is expected with the name of the library server database
	 * name.
	 *
	 * @param     props - The registry-specific properties with which to
	 *                    initialize the custom registry
	 * @exception ConnectionException
	 *                    if there is any registry-specific problem
	 **/
	public void initialize(java.util.Properties props)
		throws ConnectionException
	{
		System.out.println(">> Initialize CM WAS User Registry");
	}

	/**
	 * Checks the password of the user. This method is called to authenticate a user when the user's name
	 * and password are given.
	 *
	 * @param  userSecurityName the name of user
	 * @param  passwd the password of the user
	 * @return a valid userSecurityName. Normally this is the name of the same user whose password was checked 
	 *         but if the implementation wants to return any other valid userSecurityName in the registry
	 * 		   it can do so
	 * @exception PasswordCheckFailedException if userSecurityName/password combination does not exist 
	 *            in the registry
	 * @exception ConnectionException if there is any registry-specific problem
	 **/
	public String checkPassword(
		final String userSecurityName,
		final String passwd)
		throws PasswordCheckFailedException, ConnectionException
	{
		System.out.println(">> CM WAS User Registry - check password for user " + userSecurityName);
		//
		// Try to connect to Content Manager using the given 
		// LS database and CM schema name
		// If connect is successful, return the userid
		//		
		DKDatastoreICM dkDatastore = null;
		try
		{
			dkDatastore = ManageConnection.connect( userSecurityName, passwd, "" );
		}
		catch( ConnectionException exc )
		{
			System.out.println("Exception in ContentManagerWebSphereUserRegistry.checkPassword():");
			throw new PasswordCheckFailedException( exc );
		}			
		finally
		{
			System.out.print("Cleaning up CM connection...");
			// 
			// If a proper data store object
			// has been returned, make sure to disconnect again 
			// to avoid connection leaking
			//
			if (dkDatastore != null)
			{
				try
				{
					ManageConnection.disconnect(dkDatastore);
					System.out.println(">> successful.");
				}
				catch (Exception exc)
				{
					System.out.println(">> failed.");
					throw new ConnectionException(exc);
				}
			} else {
				System.out.println(">> not necessary.");				
			}			
		}
		return userSecurityName;
	}

	/**
	 * Maps a X.509 format certificate to a valid user in the registry.
	 * This is used to map the name in the certificate supplied by a browser
	 * to a valid userSecurityName in the registry
	 *
	 * @param     cert the X509 certificate chain
	 * @return    The mapped name of the user userSecurityName
	 * @exception CertificateMapNotSupportedException if the 
	 *            particular certificate is not supported.
	 * @exception CertificateMapFailedException if the mapping of 
	 *            the certificate fails.
	 * @exception ConnectionException if there is any registry
	 *            -specific problem
	 **/
	public String mapCertificate(X509Certificate[] cert)
		throws
			CertificateMapNotSupportedException,
			CertificateMapFailedException,
			ConnectionException
	{
		String name = null;
		X509Certificate cert1 = cert[0];
		try
		{
			System.out.println(">> CM WAS User Registry - mapCertificate");
			// map the SubjectDN in the certificate to a userID.
			name = cert1.getSubjectDN().getName();
		}
		catch (Exception ex)
		{
			throw new CertificateMapNotSupportedException(ex.getMessage(), ex);
		}

		if (!isValidUser(name))
		{
			throw new CertificateMapFailedException(
				"user: " + name + " is not valid");
		}
		return name;
	}

	/**
	 * Returns the realm of the registry.
	 *
	 * @return the realm. The realm is a registry-specific string 
	 * indicating the realm or domain for which this registry 
	 * applies. For example, for OS/400 or AIX this would be 
	 * the host name of the system whose user registry this 
	 * object represents. If null is returned by this method,
	 * realm defaults to the value of "customRealm". It is 
	 * recommended that you use your own value for realm.
	 * 
	 * @exception ConnectionException if there is any registry-
	 * specific problem
	 **/
	public String getRealm() throws ConnectionException
	{
		System.out.println(">> CM WAS User Registry - getRealm");
		return "ContentManager";
	}

	/**
	 * Gets a list of users that match a pattern in the registry.
	 * The maximum number of users returned is defined by the limit
	 * argument.
	 * This method is called by the administrative console and scripting 
	 * (command line) to make the users in the registry available for 
	 * adding them (users) to roles.
	 *
	 * @param      pattern the pattern to match. (For example, a* will 
	 *             match  all userSecurityNames starting with a)
	 * @param      limit the maximum number of users that should be
	 *             returned. This is very useful in situations where 
	 *             there are thousands of users in the registry and 
	 *             getting all of them at once is not practical. The 
	 *             default is 100. A value of 0 implies get all the
	 *             users and hence must be used with care.
	 * @return     a Result object that contains the list of users 
	 *             requested and a flag to indicate if  more users
	 *             exist.
	 * @exception  ConnectionException if there is any registry-
	 *             specificproblem
	 **/
	public Result getUsers(String pattern, int limit)
		throws ConnectionException
	{
		DKDatastoreICM dkDatastore = null;
		List allUsers = new ArrayList();
		Result result = new Result();
		int count = 0;

		try
		{
			System.out.println("CM WAS User Registry - getUsers, pattern="+pattern+", limit="+limit);
			dkDatastore = ManageConnection.connect();
			DKDatastoreAdminICM dkDataStoreAdmin =
				new DKDatastoreAdminICM(dkDatastore);
			dkUserManagement userManagement = dkDataStoreAdmin.userManagement();
			String[] users = userManagement.listUsers();
			RegExpSample regexp = new RegExpSample(pattern);
			for (int i = 0; i < users.length; i++)
			{
				String user = users[i];
				if (regexp.match(user))
				{
					if (limit == 0 || count < limit)
					{
						count++;
						allUsers.add(user);
					}
					else
					{
						result.setHasMore();
						break;
					}
				}
			}
		}
		catch (Exception exc)
		{
			System.out.println("Exception in ContentManagerWebSphereUserRegistry.getUsers():");
			exc.printStackTrace(System.out);
			throw new ConnectionException(exc);
		}
		finally
		{
			if (dkDatastore != null)
			{
				ManageConnection.disconnect(dkDatastore);
			}
		}
		result.setList(allUsers);
		return result;
	}

	/**
	 * Returns the display name for the user specified by 
	 *  userSecurityName.
	 *
	 * This method may be called only when the user information
	 * is displayed (information purposes only, for example, in 
	 * the administrative console) and hence not used in the actual 
	 * authentication or authorization purposes. If there are no 
	 * display names in the registry return null or empty string.
	 *
	 * In WebSphere Application Server 4 custom registry, if you 
	 * had a display name for the user and if it was different from the 
	 * security name, the display name was returned for the EJB 
	 * methods getCallerPrincipal() and the servlet methods
	 * getUserPrincipal() and  getRemoteUser().
	 * In WebSphere Application Server Version 5, for the same 
	 * methods, the security name will be returned by default. This 
	 * is the recommended way as the display name is not unique 
	 * and might create security holes. However, for backward 
	 * compatibility if one needs the display name to be returned 
	 * set the property WAS_UseDisplayName to true.
	 *
	 *See the InfoCenter documentation for more information.
	 *
	 * @param     userSecurityName the name of the user.
	 * @return    the display name for the user. The display 
	 *            name is a registry-specific string that 
	 *            represents a descriptive, not necessarily 
	 *            unique, name for a user. If a display name 
	 *            does not exist return null or empty string.
	 * @exception EntryNotFoundException if userSecurityName 
	 *            does not exist.
	 * @exception ConnectionException if there is any registry- 
	  *           specific problem
	 **/
	public String getUserDisplayName(String userSecurityName)
		throws ConnectionException, EntryNotFoundException
	{
		DKDatastoreICM dkDatastore = null;
		String userDisplayName = userSecurityName;

		try
		{
			System.out.println(">> CM WAS User Registry - getUserDisplayName");
			dkDatastore = ManageConnection.connect();
			DKDatastoreAdminICM dkDataStoreAdmin =
				new DKDatastoreAdminICM(dkDatastore);
			dkUserManagement userManagement = dkDataStoreAdmin.userManagement();
			dkUserDef userDef =
				userManagement.retrieveUserDef(userSecurityName);
			userDisplayName = userDef.getDescription();
		}
		catch (Exception exc)
		{
			System.out.println("Exception in ContentManagerWebSphereUserRegistry.getUserDisplayName():");
			exc.printStackTrace(System.out);
			throw new ConnectionException(exc);
		}
		finally
		{
			if (dkDatastore != null)
			{
				ManageConnection.disconnect(dkDatastore);
			}
		}

		return userDisplayName;
	}

	/**
	 * Returns the unique ID for a userSecurityName. This method is called 
	 * when creating a credential for a user.
	 *
	 * @param    userSecurityName - The name of the user.
	 * @return   The unique ID of the user. The unique ID for an user 
	 *           is the stringified form of some unique, registry-specific, 
	 *           data that serves to represent the user. For example, for 
	 *           the UNIX user registry, the unique ID for a user can be 
	 *           the UID.
	 * @exception EntryNotFoundException if userSecurityName does not 
	 *            exist.
	 * @exception ConnectionException if there is any registry-
	 *            specific problem
	 **/
	public String getUniqueUserId(String userSecurityName)
	throws ConnectionException, EntryNotFoundException
	{
		DKDatastoreICM dkDatastore = null;
		String userName = userSecurityName.toUpperCase(); // fallback 

		try
		{
			System.out.println(">> CM WAS User Registry - getUniqueUserId");
			dkDatastore = ManageConnection.connect();
			DKDatastoreAdminICM dkDataStoreAdmin = new DKDatastoreAdminICM(dkDatastore);
			dkUserManagement userManagement = dkDataStoreAdmin.userManagement();
			dkUserDef userDef = userManagement.retrieveUserDef(userSecurityName);
			userName = userDef.getName();
		}
		catch (Exception exc)
		{
			System.out.println("Exception in ContentManagerWebSphereUserRegistry.getUniqueUserId():");
			exc.printStackTrace(System.out);
			throw new ConnectionException(exc);
		}
		finally
		{
			if (dkDatastore != null)
			{
				ManageConnection.disconnect(dkDatastore);
			}
		}

		return userName;
	}

	/**
	 * Returns the name for a user given its uniqueId.
	 *
	 * @param      uniqueUserId  - The unique ID of the user.
	 * @return     The userSecurityName of the user.
	 * @exception  EntryNotFoundException if the unique user ID does not exist.
	 * @exception  ConnectionException if there is any registry-specific
	 *             problem
	 **/
	public String getUserSecurityName(String uniqueUserId)
		throws ConnectionException, EntryNotFoundException
	{
		return uniqueUserId;
	}

	/**
	 * Determines if the userSecurityName exists in the registry
	 *
	 * @param     userSecurityName - The name of the user
	 * @return    True if the user is valid; otherwise false
	 * @exception ConnectionException if there is any registry-
	 *            specific problem
	 **/
	public boolean isValidUser(String userSecurityName)
		throws ConnectionException
	{
		DKDatastoreICM dkDatastore = null;
		boolean bIsValid = false;

		try
		{
			System.out.println(">> CM WAS User Registry - isValidUser");
			dkDatastore = ManageConnection.connect();
			DKDatastoreAdminICM dkDataStoreAdmin =
				new DKDatastoreAdminICM(dkDatastore);
			dkUserManagement userManagement = dkDataStoreAdmin.userManagement();
			dkUserDef userDef =
				userManagement.retrieveUserDef(userSecurityName);
			if (userDef != null)
			{
				bIsValid = true;
			}
		}
		catch (Exception exc)
		{
			System.out.println("Exception in ContentManagerWebSphereUserRegistry.isValidUser():");
			exc.printStackTrace(System.out);
			throw new ConnectionException(exc);
		}
		finally
		{
			if (dkDatastore != null)
			{
				ManageConnection.disconnect(dkDatastore);
			}
		}
		return bIsValid;
	}

	/**
	 * Gets a list of groups that match a pattern in the registry
	 * The maximum number of groups returned is defined by the 
	 * limit argument. This method is called by administrative console
	 * and scripting (command line) to make available the groups in  
	 * the registry for adding them (groups) to roles.
	 *
	 * @param       pattern the pattern to match. (For example, a* matches 
	 *              all groupSecurityNames starting with a)
	 * @param       limit the maximum number of groups to return 
	 *              This is very useful in situations where there 
	 *              are thousands of groups in the registry and getting all 
	 *              of them at once is not practical. The default is 100. 
	 *              A value of 0 implies get all the groups and hence must 
	 *              be used with care.
	 * @return      A Result object that contains the list of groups 
	 *              requested and a flag to indicate if more groups exist.
	 * @exception ConnectionException if there is any registry-specific
	 *              problem
	 **/
	public Result getGroups(String pattern, int limit)
		throws ConnectionException
	{
		DKDatastoreICM dkDatastore = null;
		List allGroups = new ArrayList();
		Result result = new Result();
		int count = 0;

		try
		{
			System.out.println(">> CM WAS User Registry - getGroups, pattern="+pattern+", limit="+limit);
			dkDatastore = ManageConnection.connect();
			DKDatastoreAdminICM dkDataStoreAdmin =
				new DKDatastoreAdminICM(dkDatastore);
			dkUserManagement userManagement = dkDataStoreAdmin.userManagement();
			String[] groupNames = userManagement.listUserGroups();
			RegExpSample regexp = new RegExpSample(pattern);
			for (int i = 0; i < groupNames.length; i++)
			{
				String group = groupNames[i];
				if (regexp.match(group))
				{
					if (limit == 0 || count < limit)
					{
						count++;
						allGroups.add(group);
					}
					else
					{
						result.setHasMore();
						break;
					}
				}
			}
		}
		catch (Exception exc)
		{
			System.out.println("Exception in ContentManagerWebSphereUserRegistry.getGroups():");
			exc.printStackTrace(System.out);
			throw new ConnectionException(exc);
		}
		finally
		{
			if (dkDatastore != null)
			{
				ManageConnection.disconnect(dkDatastore);
			}
		}

		result.setList(allGroups);
		return result;
	}

	/**
	 * Returns the display name for the group specified by groupSecurityName.
	 * For this version of WebSphere Application Server, the only usage of  
	 * this method is by the clients (administrative console and scripting)   
	 * to present a descriptive name of the user if it exists.
	 *
	 * @param groupSecurityName the name of the group.
	 * @return  the display name for the group. The display name
	 *          is a registry-specific string that represents a  
	 *          descriptive, not necessarily unique, name for a group.  
	 *          If a display name does not exist return null or empty 
	 *          string.
	 * @exception EntryNotFoundException if groupSecurityName does 
	 *          not exist.
	 * @exception ConnectionException if there is any registry-
	 *          specific problem
	 **/
	public String getGroupDisplayName(String groupSecurityName)
		throws ConnectionException, EntryNotFoundException
	{
		DKDatastoreICM dkDatastore = null;
		String groupDisplayName = "";

		try
		{
			System.out.println(">> CM WAS User Registry - getGroupDisplayName, name="+groupSecurityName);			
			dkDatastore = ManageConnection.connect();
			DKDatastoreAdminICM dkDataStoreAdmin =
				new DKDatastoreAdminICM(dkDatastore);
			dkUserManagement userManagement = dkDataStoreAdmin.userManagement();
			dkUserGroupDef groupDef =
				userManagement.retrieveUserGroupDef(groupSecurityName);
			groupDisplayName = groupDef.getDescription();
		}
		catch (Exception exc)
		{
			System.out.println("Exception in ContentManagerWebSphereUserRegistry.getGroupDisplayName():");
			exc.printStackTrace(System.out);
			throw new ConnectionException(exc);
		}
		finally
		{
			if (dkDatastore != null)
			{
				ManageConnection.disconnect(dkDatastore);
			}
		}

		return groupDisplayName;
	}

	/**
	 * Returns the Unique ID for a group.
	
	 * @param     groupSecurityName the name of the group.
	 * @return    The unique ID of the group. The unique ID for
	 *            a group is the stringified form of some unique,
	 *            registry-specific, data that serves to represent
	 *            the group. For example, for the UNIX user registry,
	 *            the unique ID might be the GID.
	 * @exception EntryNotFoundException if groupSecurityName does 
	 *            not exist.
	 * @exception ConnectionException if there is any registry-
	 *            specific problem
	 * @exception RemoteException as this extends java.rmi.Remote
	 **/
	public String getUniqueGroupId(String groupSecurityName)
		throws ConnectionException, EntryNotFoundException
	{
		return groupSecurityName;
	}

	/**
	 * Returns the Unique IDs for all the groups that contain the UniqueId 
	 * of a user. Called during creation of a user's credential.
	 *
	 * @param     uniqueUserId the unique ID of the user.
	 * @return    A list of all the group unique IDs that the unique user 
	 *            ID belongs to. The unique ID for an entry is the 
	 *            stringified form of some unique, registry-specific, data 
	 *            that serves to represent the entry.  For example, for the
	 *            UNIX user registry, the unique ID for a group might be 
	 *            the GID and the Unique ID for the user might be the UID.
	 * @exception EntryNotFoundException if uniqueUserId does not exist.
	 * @exception ConnectionException if there is any registry-specific
	 *            problem
	 **/
	public List getUniqueGroupIds(String uniqueUserId)
		throws ConnectionException, EntryNotFoundException
	{
		// Loop over all groups and check whether the userid is part of the group			
		List uniqueGrpIds = new ArrayList();
		DKDatastoreICM dkDatastore = null;
		try
		{
			System.out.println(">> CM WAS User Registry - getUniqueGroupIds, user="+uniqueUserId);			
			dkDatastore = ManageConnection.connect();
			DKDatastoreAdminICM dkDataStoreAdmin =
				new DKDatastoreAdminICM(dkDatastore);
			dkUserManagement userManagement = dkDataStoreAdmin.userManagement();
			dkIterator it = userManagement.listUserGroupDefs().createIterator();
			while (it.more())
			{
				dkUserGroupDef groupDef = (dkUserGroupDef) it.next();
				String[] users = groupDef.listUserNames();
				for (int i = 0; i < users.length; i++)
				{
					if (uniqueUserId.equals(users[i]))
					{
						uniqueGrpIds.add(groupDef.getName());
						System.out.println(">> ..."+groupDef.getName());			
						break;
					}
				}
			}
		}
		catch (Exception exc)
		{
			System.out.println("Exception in ContentManagerWebSphereUserRegistry.getUniqueGroupIds():");
			exc.printStackTrace(System.out);
			throw new ConnectionException(exc);
		}
		finally
		{
			if (dkDatastore != null)
			{
				ManageConnection.disconnect(dkDatastore);
			}
		}
		return uniqueGrpIds;
	}

	/**
	 * Returns the name for a group given its uniqueId.
	 *
	 * @param     uniqueGroupId the unique ID of the group.
	 * @return    The name of the group.
	 * @exception EntryNotFoundException if the uniqueGroupId does 
	 *            not exist.
	 * @exception ConnectionException if there is any registry-
	 *            specific problem
	 **/
	public String getGroupSecurityName(String uniqueGroupId)
		throws ConnectionException, EntryNotFoundException
	{
		return uniqueGroupId;
	}

	/**
	 * Determines if the groupSecurityName exists in the registry
	 *
	 * @param     groupSecurityName the name of the group
	 * @return    True if the groups exists; otherwise false
	 * @exception ConnectionException if there is any registry-
	 *            specific problem
	 **/
	public boolean isValidGroup(String groupSecurityName)
		throws ConnectionException
	{
		DKDatastoreICM dkDatastore = null;
		boolean bIsValid = false;

		try
		{
			dkDatastore = ManageConnection.connect();
			DKDatastoreAdminICM dkDataStoreAdmin =
				new DKDatastoreAdminICM(dkDatastore);
			dkUserManagement userManagement = dkDataStoreAdmin.userManagement();
			dkUserGroupDef groupDef =
				userManagement.retrieveUserGroupDef(groupSecurityName);
			if (groupDef != null)
			{
				bIsValid = true;
			}
		}
		catch (Exception exc)
		{
			System.out.println("Exception in ContentManagerWebSphereUserRegistry.isValidGroup():");
			exc.printStackTrace(System.out);
			throw new ConnectionException(exc);
		}
		finally
		{
			if (dkDatastore != null)
			{
				ManageConnection.disconnect(dkDatastore);
			}
		}
		return bIsValid;
	}

	/**
	 * Returns the securityNames of all the groups that contain the user
	 *
	 * This method is called by the administrative console and scripting 
	 * (command line) to verify the user entered for RunAsRole mapping  
	 * belongs to that role in the roles to user mapping. Initially, the 
	 * check is done to see  if the role contains the user. If the role does 
	 * not contain the user explicitly, this method is called to get the groups 
	 * that this user belongs to so that check can be made on the groups that 
	 * the role contains.
	 *
	 * @param     userName the name of the user
	 * @return    A list of all the group securityNames that the user
	 *            belongs to.
	 * @exception EntryNotFoundException if user does not exist.
	 * @exception ConnectionException if there is any registry-
	 *            specific problem
	 * @exception RemoteException as this extends the java.rmi.Remote
	 *            interface 
	 **/
	public List getGroupsForUser(String userName)
		throws ConnectionException, EntryNotFoundException
	{
		return getUniqueGroupIds(userName);
	}

	/**
	 * Gets a list of users in a group.  
	 *
	 * The maximum number of users returned is defined by the 
	 * limit argument.
	 *	
	 * This method is being used by process choreographer
	 * when staff assignments are modeled using groups.
	 *
	 * In rare situations, if you are working with a registry where  
	 * getting all the users from any of your groups is not practical   
	 * (for example if there are a large number of users) you can throw
	 * the NotImplementedException for that particular group. Make sure   
	 * that if the process choreographer is installed (or if installed later) 
	 * the staff assignments are not modeled using these particular groups.
	 * If there is no concern about returning the users from groups 
	 * in the registry it is recommended that this method be implemented
	 * without throwing the NotImplemented exception.
	 * @param         groupSecurityName the name of the group
	 * @param         limit the maximum number of users that should be 
	 *                returned. This is very useful in situations where there 
	 *                are lot of users in the registry and getting all of  
	 *                them at once is not practical. A value of 0 implies   
	 *                get all the users and hence must be used with care. 
	 * @return        A result object that contains the list of users
	 *                requested and a flag to indicate if more users exist.
	 * @deprecated    This method will be deprecated in future.
	 * @exception     NotImplementedException throw this exception in rare 
	 *                situations if it is not practical to get this information   
	 *                for any of the group or groups from the registry.
	 * @exception     EntryNotFoundException if the group does not exist in 
	 *                the registry
	 * @exception     ConnectionException if there is any registry-specific 
	 *                problem
	 **/
	public Result getUsersForGroup(String groupSecurityName, int limit)
		throws
			NotImplementedException,
			EntryNotFoundException,
			ConnectionException
	{

		DKDatastoreICM dkDatastore = null;
		ArrayList usersForGroup = new ArrayList();
		Result result = new Result();

		try
		{
			System.out.println(">> CM WAS User Registry - getUsersForGroup, group="+groupSecurityName+", limit="+limit);
			dkDatastore = ManageConnection.connect();
			DKDatastoreAdminICM dkDataStoreAdmin =
				new DKDatastoreAdminICM(dkDatastore);
			dkUserManagement userManagement = dkDataStoreAdmin.userManagement();
			dkUserGroupDef groupDef =
				userManagement.retrieveUserGroupDef(groupSecurityName);
			if (groupDef == null)
			{
				throw new EntryNotFoundException(groupSecurityName);
			}
			String[] users = groupDef.listUserNames();
			if (limit == 0 || limit > users.length)
			{
				limit = users.length;
			}
			else
			{
				result.setHasMore();
			}
			for (int i = 0; i < limit; i++)
			{
				usersForGroup.add(users[i]);
			}
		}
		catch (Exception exc)
		{
			System.out.println("Exception in ContentManagerWebSphereUserRegistry.getUsersForGroup():");
			exc.printStackTrace(System.out);
			throw new ConnectionException(exc);
		}
		finally
		{
			if (dkDatastore != null)
			{
				ManageConnection.disconnect(dkDatastore);
			}
		}
		result.setList(usersForGroup);
		return result;
	}

	/**
	 * This method is implemented internally by the WebSphere Application 
	 * Server code in this release. This method is not called for the custom 
	 * registry implementations for this release. Return null in the 
	 * implementation.
	 *
	 **/
	public com.ibm.websphere.security.cred.WSCredential createCredential(
		String userSecurityName)
		throws
			ConnectionException,
			NotImplementedException,
			EntryNotFoundException
	{

		// This method is not called.
		return null;
	}	
}

// This code is taken from the sample in the WebSphere Info Center
// http://publib.boulder.ibm.com/infocenter/wasinfo/v5r1//index.jsp?topic=/com.ibm.wasee.doc/info/ee/ae/rsec_frsjf502.html
//----------------------------------------------------------------------
// The program provides the Regular Expression implementation 
// used in the sample for the custom user registry (FileRegistrySample). 
// The pattern matching in the sample uses this program to search for the 
// pattern (for users and groups).
//----------------------------------------------------------------------
class RegExpSample
{

	private boolean match(String s, int i, int j, int k)
	{
		for (; k < expr.length; k++)
			label0 : {
			Object obj = expr[k];
			if (obj == STAR)
			{
				if (++k >= expr.length)
					return true;
				if (expr[k] instanceof String)
				{
					String s1 = (String) expr[k++];
					int l = s1.length();
					for (;(i = s.indexOf(s1, i)) >= 0; i++)
						if (match(s, i + l, j, k))
							return true;

					return false;
				}
				for (; i < j; i++)
					if (match(s, i, j, k))
						return true;

				return false;
			}
			if (obj == ANY)
			{
				if (++i > j)
					return false;
				break label0;
			}
			if (obj instanceof char[][])
			{
				if (i >= j)
					return false;
				char c = s.charAt(i++);
				char ac[][] = (char[][]) obj;
				if (ac[0] == NOT)
				{
					for (int j1 = 1; j1 < ac.length; j1++)
						if (ac[j1][0] <= c && c <= ac[j1][1])
							return false;

					break label0;
				}
				for (int k1 = 0; k1 < ac.length; k1++)
					if (ac[k1][0] <= c && c <= ac[k1][1])
						break label0;

				return false;
			}
			if (obj instanceof String)
			{
				String s2 = (String) obj;
				int i1 = s2.length();
				if (!s.regionMatches(i, s2, 0, i1))
					return false;
				i += i1;
			}
		}

		return i == j;
	}

	public boolean match(String s)
	{
		return match(s, 0, s.length(), 0);
	}

	public boolean match(String s, int i, int j)
	{
		return match(s, i, j, 0);
	}

  public RegExpSample(String s)
	{
		Vector vector = new Vector();
		int i = s.length();
		StringBuffer stringbuffer = null;
		Object obj = null;
		for (int j = 0; j < i; j++)
		{
			char c = s.charAt(j);
			switch (c)
			{
				case 63 : /* '?' */
					obj = ANY;
					break;

				case 42 : /* '*' */
					obj = STAR;
					break;

				case 91 : /* '[' */
					int k = ++j;
					Vector vector1 = new Vector();
					for (; j < i; j++)
					{
						c = s.charAt(j);
						if (j == k && c == '^')
						{
							vector1.addElement(NOT);
							continue;
						}
						if (c == '\\')
						{
							if (j + 1 < i)
								c = s.charAt(++j);
						}
						else if (c == ']')
							break;
						char c1 = c;
						if (j + 2 < i && s.charAt(j + 1) == '-')
							c1 = s.charAt(j += 2);
						char ac1[] = { c, c1 };
						vector1.addElement(ac1);
					}

					char ac[][] = new char[vector1.size()][];
					vector1.copyInto(ac);
					obj = ac;
					break;

				case 92 : /* '\\' */
					if (j + 1 < i)
						c = s.charAt(++j);
					break;

			}
			if (obj != null)
			{
				if (stringbuffer != null)
				{
					vector.addElement(stringbuffer.toString());
					stringbuffer = null;
				}
				vector.addElement(obj);
				obj = null;
			}
			else
			{
				if (stringbuffer == null)
					stringbuffer = new StringBuffer();
				stringbuffer.append(c);
			}
		}

		if (stringbuffer != null)
			vector.addElement(stringbuffer.toString());
		expr = new Object[vector.size()];
		vector.copyInto(expr);
	}

	static final char NOT[] = new char[2];
	static final Integer ANY = new Integer(0);
	static final Integer STAR = new Integer(1);
	Object expr[];

}
