/*
@copyright(disclaimer) 
Licensed Materials - Property of IBM
IBM DB2 Content Manager Enterprise Edition V8  (program number 5724-B19)
IBM DB2 Content Manager Express Edition V8 (program number 5724-F73)
IBM DB2 Content Manager for z/OS V8 (program number 5697-H60)
(c ) Copyright IBM Corp. 1994,  2005. All Rights Reserved.

US Government Users Restricted Rights                                  
Use, duplication or disclosure restricted by GSA ADP Schedule          
Contract with IBM Corp.     

DISCLAIMER OF WARRANTIES :                                             
                                                                       
Permission is granted to copy and modify this  Sample code, and to           
distribute modified versions provided that both the copyright        
notice, and this permission notice and warranty disclaimer  appear in all copies and modified  
versions. 

This software is provided "AS IS."  IBM and its Suppliers and Licensors expressly disclaim all
warranties, whether  EXPRESS OR IMPLIED,  INCLUDING ANY IMPLIED WARRANTY OF MERCHANTABILITY,
FITNESS FOR A PARTICULAR PURPOSE OR WARRANTY OF  NON-INFRINGEMENT.  IBM AND ITS SUPPLIERS AND
LICENSORS SHALL NOT BE LIABLE FOR ANY DAMAGES SUFFERED BY LICENSEE THAT RESULT FROM USE OR
DISTRIBUTION OF THE SOFTWARE OR THE COMBINATION OF THE SOFTWARE WITH ANY OTHER CODE.
IN NO EVENT WILL IBM OR ITS SUPPLIERS  AND LICENSORS BE LIABLE FOR ANY LOST REVENUE, PROFIT OR
DATA, OR FOR DIRECT, INDIRECT, SPECIAL, CONSEQUENTIAL, INCIDENTAL OR PUNITIVE DAMAGES, HOWEVER
CAUSED AND REGARDLESS OF THE THEORY OF LIABILITY, ARISING OUT OF THE USE OF OR INABILITY TO USE
SOFTWARE, EVEN IF IBM HAS BEEN ADVISED OF THE POSSIBILITY OF SUCH DAMAGES.                          
                                                                 
@endCopyright     
*/

package com.ibm.bpe.cm.servlets;
import java.io.IOException;
import java.util.Enumeration;

import javax.servlet.Servlet;
import javax.servlet.ServletException;
import javax.servlet.http.HttpServlet;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import javax.servlet.http.HttpSession;

import com.ibm.bpe.cm.CMBPCConstants;
import com.ibm.bpe.cm.utils.Helpers;
import com.ibm.task.api.HumanTaskManager;
import com.ibm.task.api.HumanTaskManagerHome;
import com.ibm.task.api.Task;
import commonj.sdo.DataObject;

/**
* This servlet sets the state of the current activity instance to completed.
*/

public class CompleteWorkServlet extends HttpServlet implements Servlet {

	public void doGet(HttpServletRequest request,
					  HttpServletResponse response) throws ServletException, IOException {
		doPost(request, response);
	}

	public void doPost(HttpServletRequest request, HttpServletResponse response) throws ServletException, IOException {
		String tkiidString   = request.getParameter("TKIID");	
		HttpSession session = request.getSession();
		String errorTarget  = null;   // in case of error this is set to an URL to forward the request to
		
		try {
			
			// fetch the process bean from the session
		    Helpers.validateConnection(request);
			HumanTaskManagerHome htmHome = (HumanTaskManagerHome)session.getAttribute("HTMHome");
			HumanTaskManager htm = htmHome.create();
			
			Task task = htm.getTask(tkiidString);
			int taskState = task.getState();

			// fetch instance data and state based on the activity instance id
//			ActivityInstanceData activity = bps.getActivityInstance(tkiidString);
//			String activityName = activity.getName();
//			int activityState = activity.getExecutionState();
			
			if (taskState == Task.STATE_CLAIMED) { // the activity instance needs to be in state claimed to be completed
				// complete the work item
				if (request.getParameter("CANCEL") == null) {
					// compose the output message of this activity
				    com.ibm.task.api.ClientObjectWrapper cow = htm.createOutputMessage(tkiidString);
				    DataObject outDO = (DataObject)cow.getObject();
					if (outDO != null) {
						DataObject responseDO = outDO.createDataObject(CMBPCConstants.BOATT_TASK_RESPONSE);

						// This generic approach only works for attributes of type string. It assumes that the name postfix
						// of a msgpart.xxx input form element (xxx) is identical to the part name used in the WSDL that defines
						// the output message and that the value instantiates the message part.
						// If names are different or non-string-type parts are used, the following code section needs to be
						// replaced by an activity-specific implementation
						Enumeration attributes = request.getParameterNames();
						String key, value, internalKey;
						int internalKeyPos = 0;

						// iterate over attributes with prefix "msgpart." and add the corresponding key/value pairs to the
						// output message of this staff activity
						while (attributes.hasMoreElements()) {
							key = (String) attributes.nextElement();
							value = (String) request.getParameter(key);
							if (key.startsWith("msgpart.") == true && value != null) {
								key = key.substring(8);

								responseDO.setString(key, value);
							}
						}
					} // outDO != null
				
					// set the activity instance state to COMPLETED
					htm.complete(tkiidString, cow);
					System.out.println("-> Work item completed");
				}
				// cancel work item
				else {
						htm.cancelClaim(tkiidString);
						System.out.println("-> Work item canceled");
		    	}			
			}
			else {                        // work item is not in state CLAIMED
				errorTarget = "listWorkItems.jsp?msg=wrongState";  // -> it can not be completed or canceled: notify user
			}
		}
		catch (Exception e) {
			e.printStackTrace();
			errorTarget = "error.jsp";
		}
		if (errorTarget == null) {
			String workListIdString = (String)session.getAttribute("WLID");
			if (workListIdString == null) workListIdString = "0";
			response.sendRedirect("listWorkItems.jsp?WLID=" + workListIdString);
		}
		else
			request.getRequestDispatcher(errorTarget).forward(request, response);
 }
 
} // CompleteWorkServlet
