/*
@copyright(disclaimer) 
Licensed Materials - Property of IBM
IBM DB2 Content Manager Enterprise Edition V8  (program number 5724-B19)
IBM DB2 Content Manager Express Edition V8 (program number 5724-F73)
IBM DB2 Content Manager for z/OS V8 (program number 5697-H60)
(c ) Copyright IBM Corp. 1994,  2005. All Rights Reserved.

US Government Users Restricted Rights                                  
Use, duplication or disclosure restricted by GSA ADP Schedule          
Contract with IBM Corp.     

DISCLAIMER OF WARRANTIES :                                             
                                                                       
Permission is granted to copy and modify this  Sample code, and to           
distribute modified versions provided that both the copyright        
notice, and this permission notice and warranty disclaimer  appear in all copies and modified  
versions. 

This software is provided "AS IS."  IBM and its Suppliers and Licensors expressly disclaim all
warranties, whether  EXPRESS OR IMPLIED,  INCLUDING ANY IMPLIED WARRANTY OF MERCHANTABILITY,
FITNESS FOR A PARTICULAR PURPOSE OR WARRANTY OF  NON-INFRINGEMENT.  IBM AND ITS SUPPLIERS AND
LICENSORS SHALL NOT BE LIABLE FOR ANY DAMAGES SUFFERED BY LICENSEE THAT RESULT FROM USE OR
DISTRIBUTION OF THE SOFTWARE OR THE COMBINATION OF THE SOFTWARE WITH ANY OTHER CODE.
IN NO EVENT WILL IBM OR ITS SUPPLIERS  AND LICENSORS BE LIABLE FOR ANY LOST REVENUE, PROFIT OR
DATA, OR FOR DIRECT, INDIRECT, SPECIAL, CONSEQUENTIAL, INCIDENTAL OR PUNITIVE DAMAGES, HOWEVER
CAUSED AND REGARDLESS OF THE THEORY OF LIABILITY, ARISING OUT OF THE USE OF OR INABILITY TO USE
SOFTWARE, EVEN IF IBM HAS BEEN ADVISED OF THE POSSIBILITY OF SUCH DAMAGES.                          
                                                                 
@endCopyright     
*/

package com.ibm.bpe.cm.servlets;

import java.io.IOException;

import javax.servlet.Servlet;
import javax.servlet.ServletException;
import javax.servlet.http.HttpServlet;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import javax.servlet.http.HttpSession;

import com.ibm.bpe.api.BusinessFlowManager;
import com.ibm.bpe.api.BusinessFlowManagerHome;
import com.ibm.bpe.api.ClientObjectWrapper;
import com.ibm.bpe.api.ProcessTemplateData;
import com.ibm.bpe.cm.CMBPCConstants;
import com.ibm.bpe.cm.utils.Helpers;
import com.ibm.mm.beans.CMBBaseConstant;
import com.ibm.mm.beans.CMBConnection;
import com.ibm.mm.beans.CMBItem;
import com.ibm.mm.beans.CMBQueryService;
import com.ibm.mm.beans.CMBSearchResults;
import com.ibm.mm.sdk.common.DKPidICM;
import commonj.sdo.DataObject;

/**
 * This servlet runs an XPath query on the library server based on the query specified by the 'query' parameter.
 * For each element of the result set it creates a process instance the input message of which is initalized with
 * the PID of this item.
 */

public class StartProcessInstancesServlet extends HttpServlet implements Servlet {
	
	public void doGet(HttpServletRequest request, HttpServletResponse response) throws ServletException, IOException {
		doPost(request, response);
	}

	public void doPost(HttpServletRequest request, HttpServletResponse response) throws ServletException, IOException {
		String errorTarget = null;   // in case of error this is set to an URL to forward the request to
		int numHits		   = 0;

		try {
			// locate the process bean and connection in the session context
  			HttpSession session = request.getSession();
			Helpers.validateConnection(request);
			BusinessFlowManagerHome processHome = (BusinessFlowManagerHome)session.getAttribute("ProcessHome");
			BusinessFlowManager bp = processHome.create();
			CMBConnection connection = (CMBConnection)session.getAttribute("connection");
			String processName = request.getParameter("processName");

			System.out.println(">> Starting instance of process: " + processName);
			
			ProcessTemplateData ptdata = bp.getProcessTemplate(processName);
    
			// Set up the Content Manager query service
			String query = request.getParameter("query");
			CMBQueryService queryService = connection.getQueryService();
			short queryType = CMBBaseConstant.CMB_QS_TYPE_XPATH;
			queryService.setQueryString(query, queryType);
			queryService.setAsynchSearch(false);
    
			// Search for items matching the query
			System.out.println("-> Running query [" + query + "]");
			queryService.runQuery();
    
			// Fetch results
			CMBSearchResults searchResults = new CMBSearchResults();
			searchResults.setConnection(connection);
			searchResults.newResults(queryService.getResults());
              
			// Iterate over results and create process instances for each item in the result list
			numHits = searchResults.getCount();
			for (int i = 0; i < numHits; i++) {
				ClientObjectWrapper cow = bp.createMessage(ptdata.getID(), ptdata.getInputMessageTypeName());
				CMBItem item = searchResults.getItem(i);
				DKPidICM pid = new DKPidICM(item.getPidString());
				
				// get the process input DO and sets values to the itemId, componentId, versionNo of the retrieved item
				DataObject processInputDO = (DataObject)cow.getObject();
				processInputDO.setString(CMBPCConstants.BOATT_PROCESS_INTERFACE_ITEMID, pid.getItemId());
				processInputDO.setString(CMBPCConstants.BOATT_PROCESS_INTERFACE_COMPONENTID, pid.getComponentId());
				processInputDO.setString(CMBPCConstants.BOATT_PROCESS_INTERFACE_VERSIONNO, pid.getVersionNumber());
				
				// start the business process instance
				bp.initiate(processName, cow);
			}
			System.out.println("-> " + numHits + " process instances created.");
		}
 		catch (Exception e) {
	  		e.printStackTrace();
			errorTarget = "error.jsp";
		}
		if (errorTarget == null)
			response.sendRedirect("startProcessInstances.jsp?numHits=" + numHits);
		else
			request.getRequestDispatcher(errorTarget).forward(request, response);
 	}
 	
} // StartProcessInstancesServlet
