/*
@copyright(disclaimer) 
Licensed Materials - Property of IBM
IBM DB2 Content Manager Enterprise Edition V8  (program number 5724-B19)
IBM DB2 Content Manager Express Edition V8 (program number 5724-F73)
IBM DB2 Content Manager for z/OS V8 (program number 5697-H60)
(c ) Copyright IBM Corp. 1994,  2005. All Rights Reserved.

US Government Users Restricted Rights                                  
Use, duplication or disclosure restricted by GSA ADP Schedule          
Contract with IBM Corp.     

DISCLAIMER OF WARRANTIES :                                          
                                                                       
Permission is granted to copy and modify this  Sample code, and to           
distribute modified versions provided that both the copyright        
notice, and this permission notice and warranty disclaimer  appear in all copies and modified  
versions. 

This software is provided "AS IS."  IBM and its Suppliers and Licensors expressly disclaim all
warranties, whether  EXPRESS OR IMPLIED,  INCLUDING ANY IMPLIED WARRANTY OF MERCHANTABILITY,
FITNESS FOR A PARTICULAR PURPOSE OR WARRANTY OF  NON-INFRINGEMENT.  IBM AND ITS SUPPLIERS AND
LICENSORS SHALL NOT BE LIABLE FOR ANY DAMAGES SUFFERED BY LICENSEE THAT RESULT FROM USE OR
DISTRIBUTION OF THE SOFTWARE OR THE COMBINATION OF THE SOFTWARE WITH ANY OTHER CODE.
IN NO EVENT WILL IBM OR ITS SUPPLIERS  AND LICENSORS BE LIABLE FOR ANY LOST REVENUE, PROFIT OR
DATA, OR FOR DIRECT, INDIRECT, SPECIAL, CONSEQUENTIAL, INCIDENTAL OR PUNITIVE DAMAGES, HOWEVER
CAUSED AND REGARDLESS OF THE THEORY OF LIABILITY, ARISING OUT OF THE USE OF OR INABILITY TO USE
SOFTWARE, EVEN IF IBM HAS BEEN ADVISED OF THE POSSIBILITY OF SUCH DAMAGES.                          
                                                                 
@endCopyright     
*/

package com.ibm.bpe.cm.servlets;
import java.io.IOException;
import java.util.Enumeration;

import javax.servlet.Servlet;
import javax.servlet.ServletException;
import javax.servlet.http.HttpServlet;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import javax.servlet.http.HttpSession;

import com.ibm.bpe.cm.utils.Helpers;
import com.ibm.bpe.cm.utils.ProcessData;
import com.ibm.mm.beans.CMBBaseConstant;
import com.ibm.mm.beans.CMBConnection;
import com.ibm.mm.beans.CMBDataManagement;
import com.ibm.mm.beans.CMBException;
import com.ibm.mm.beans.CMBItem;

/**
 * This servlet serves one of the following two purposes:
 * a if the parameter 'ITEMPID' is empty this servlet creates an instance of the item type 'entityName', initializes
 *   its attribute values with the values of attr.XXX parameters.
 * b if a valid 'ITEMPID' is specified this servlet retrieves the corresponding item and updates its attribute values with
 *   the values of the attr.XXX parameters.
 * Names of attributes to be updated are provided by parameters of type "attribute.[attribute name]" or
 * "attr.[child component name].intAttr.[attribute name]"
 * for attributes in child components.
 * Note: Only one level of the parent-child hierarchy is supported.
 */

public class UpdateAttributesServlet extends HttpServlet implements Servlet {
	
	public void doGet(HttpServletRequest request, HttpServletResponse response) throws ServletException, IOException {
		doPost(request, response);
	}

	public void doPost(HttpServletRequest request, HttpServletResponse response) throws ServletException, IOException {
		CMBDataManagement dataMgmt;
		CMBItem itemBean   = null;
		String tkiidString  = request.getParameter("TKIID");
		if (tkiidString == null) tkiidString = "";
		String processItemPid   = request.getParameter("PIPID");
		if (processItemPid == null) processItemPid = "";
		String itemPid = request.getParameter("ITEMPID");
		String entityName  = request.getParameter("entityName");
		
		String targetPage = null;   // in case of error this is set to an URL to forward the request to
	
		if (request.getParameter("mode") == null) {    // if (mode = nocreate) pass the request on to updateAttributes.jsp
			
			// determine if a new item is to be created or if an existing one is to be updated	
			boolean createNewItem = (itemPid.length() == 0);

			try {
				HttpSession session = request.getSession();
				Helpers.validateConnection(request);
				CMBConnection connection = (CMBConnection) session.getAttribute("connection");
				
				if (createNewItem) {  // create a new item

					dataMgmt = connection.getDataManagement();

					// create and initialize an instance of CMBItem to become the new folder member
					itemBean = new CMBItem();
					itemBean.setEntityName(entityName);
					itemBean.setItemType(CMBBaseConstant.CMB_TYPE_DOCUMENT);
					itemBean.setConnection(connection);
					itemBean.setServerType(ProcessData.CM_DATASTORE_TYPE);  // folderItem.getServerType() can only be used if a folder is available
					itemBean.setServerName(ProcessData.CM_DATASTORE_NAME);  // folderItem.getServerName() "
																			// both can not be guaranteed so we use the process definitions
				}

				else {				  // modify an existing item
					dataMgmt = Helpers.setupDataBean(request, itemPid, true);
					dataMgmt.checkOut();
					itemBean = dataMgmt.getDataObject();
				}
			
				// Access all request parameters and identify those containing attribute values.
				// The format of request parameter names for attributes is attribute.<attrname> for simple attributes
				// and attribute.<childname>.intAttr.<attrname> for attributes of child components.

				Enumeration attributes = request.getParameterNames();
				String key, value, internalKey;
				short type;
				int endVal, index;
				int internalKeyPos = 0;

				// iterate over all available attributes
				while (attributes.hasMoreElements()) {
					key = (String) attributes.nextElement();
					value = (String) request.getParameter(key);

					// identify all request parameters that have the prefix attr.XXX
					if (key.startsWith("attr.") == true && value != null) { // request parameter contains data for an attribute
						key = key.substring(5);
						endVal = key.indexOf('.');
						type = (new Short(key.substring(0, endVal))).shortValue(); // adjust offset
						key = key.substring(endVal + 1);
						internalKeyPos = key.indexOf(".intAttr.");

						if (internalKeyPos < 0) {  // the attribute does not refer to a child component
							if (createNewItem)     // new item: initialize attribute
								itemBean.addAttr(key, value, (new Integer(type)).shortValue());
							else                   // update item: update attribute
								itemBean.setAttrValue(key, value);
						}
		  				else {                    // the attribute refers to a child component
		  					
							// get attribute name from parameter value
							internalKey = key.substring(internalKeyPos + 9);
							key = key.substring(0, internalKeyPos);
							endVal = internalKey.indexOf('.');
							index = (new Integer(internalKey.substring(0, endVal))).intValue(); // adjust offset
							
							// get attribute type from parameter value
							internalKey = internalKey.substring(endVal + 1);
							endVal = internalKey.indexOf('.');
							type = (new Short(internalKey.substring(0, endVal))).shortValue(); // adjust offset
							internalKey = internalKey.substring(endVal + 1);

							// create new or update existing child attribute
							CMBItem[] attrItems;
							CMBItem itemInAttribute;
							try {						
								attrItems = itemBean.getAttrItems(key);
							}
							catch (CMBException e) {   // no child component list for this child attribute: ensure it will be created
								attrItems = null;
							}

							// child attribute with this name does not exist yet: create and initialize
							if (attrItems == null || index == attrItems.length) {
								itemInAttribute = new CMBItem();
								itemInAttribute.setEntityName(key);
								itemInAttribute.setItemType(CMBBaseConstant.CMB_TYPE_DOCUMENT);
								itemInAttribute.setConnection(connection);
								itemInAttribute.setServerType(ProcessData.CM_DATASTORE_TYPE);  // folderItem.getServerType() can be used if folder is available
								itemInAttribute.setServerName(ProcessData.CM_DATASTORE_NAME);  // folderItem.getServerName() "
								
								if (attrItems == null) { // child attribute list does not exist yet: create, add new item and instantiate child attribute
									attrItems = new CMBItem[]{ itemInAttribute }; 
									itemBean.addAttr(key, attrItems, CMBBaseConstant.CMB_DATATYPE_ITEM_COLLECTION);
								} 
								else                     // add item to existing child component list
									itemBean.addAttrItem(key, itemInAttribute);
							}
							else {                       // retrieve existing child attribute
								itemInAttribute = attrItems[index];
							}

							if (itemInAttribute.isAttr(internalKey))							 
								itemInAttribute.setAttrValue(internalKey, value);  // child attribute exists: update
							else
								itemInAttribute.addAttr(internalKey, value, type); // child attribute does not exist: add
							
							
						} // end branch for child components
					} // end branch for attributes
				} // end loop over requester parameters

				try {
					if (createNewItem) {        // create new item
						dataMgmt.setDataObject(itemBean);
						dataMgmt.createItem(entityName);
						System.out.println("-> Item created");
						itemPid = itemBean.getPidString(); 
					}
					else {						// update existing item
						dataMgmt.updateItem();
						dataMgmt.checkIn();
						System.out.println("-> Item updated");
	   				}
				}
				catch (CMBException invalidAttrValue) {  			  // this triggers a notification window that informs the user about
					invalidAttrValue.printStackTrace();
					targetPage = "updateAttributes.jsp?msg=invalid"; // an invalid attribute value; this can be due to the wrong value type
				}  			                             			  // e.g. String instead of integer, ... a range violation (> max), ....
			}
	   		
			catch (Exception e) {
				e.printStackTrace();
				targetPage = "error.jsp";
			}
			
		}  // if (mode = nocreate) - pass the request on to updateAttributes.jsp
		else
			targetPage = "updateAttributes.jsp";
		

		if (targetPage == null) {  // forward the request to ensure the attributes are still available on the request
			response.sendRedirect("updateAttributes.jsp?TKIID=" + tkiidString +
													  "&PIPID=" + processItemPid + "&ITEMPID=" + itemPid +
													  "&entityName=" + entityName);
		}
		else {
			request.getRequestDispatcher(targetPage).forward(request, response);
		}
	}
	
} // UpdateAttributesServlet
