/*
@copyright(disclaimer) 
Licensed Materials - Property of IBM
IBM DB2 Content Manager Enterprise Edition V8  (program number 5724-B19)
IBM DB2 Content Manager Express Edition V8 (program number 5724-F73)
IBM DB2 Content Manager for z/OS V8 (program number 5697-H60)
(c ) Copyright IBM Corp. 1994,  2005. All Rights Reserved.

US Government Users Restricted Rights                                  
Use, duplication or disclosure restricted by GSA ADP Schedule          
Contract with IBM Corp.     

DISCLAIMER OF WARRANTIES :                                             
                                                                       
Permission is granted to copy and modify this  Sample code, and to           
distribute modified versions provided that both the copyright        
notice, and this permission notice and warranty disclaimer  appear in all copies and modified  
versions. 

This software is provided "AS IS."  IBM and its Suppliers and Licensors expressly disclaim all
warranties, whether  EXPRESS OR IMPLIED,  INCLUDING ANY IMPLIED WARRANTY OF MERCHANTABILITY,
FITNESS FOR A PARTICULAR PURPOSE OR WARRANTY OF  NON-INFRINGEMENT.  IBM AND ITS SUPPLIERS AND
LICENSORS SHALL NOT BE LIABLE FOR ANY DAMAGES SUFFERED BY LICENSEE THAT RESULT FROM USE OR
DISTRIBUTION OF THE SOFTWARE OR THE COMBINATION OF THE SOFTWARE WITH ANY OTHER CODE.
IN NO EVENT WILL IBM OR ITS SUPPLIERS  AND LICENSORS BE LIABLE FOR ANY LOST REVENUE, PROFIT OR
DATA, OR FOR DIRECT, INDIRECT, SPECIAL, CONSEQUENTIAL, INCIDENTAL OR PUNITIVE DAMAGES, HOWEVER
CAUSED AND REGARDLESS OF THE THEORY OF LIABILITY, ARISING OUT OF THE USE OF OR INABILITY TO USE
SOFTWARE, EVEN IF IBM HAS BEEN ADVISED OF THE POSSIBILITY OF SUCH DAMAGES.                          
                                                                 
@endCopyright     
*/

package com.ibm.bpe.cm.utils;

import commonj.sdo.DataObject;

/**
 * This class contains constants that are specific to the client environment and may need to be
 * adapted to a specific CM installation, process, or content model.
 * 
 */

public class ProcessData
{
	
 /**
  * The following constants are specific to the sample process. They may need to be adjusted
  * if the client is to be used with a different process / in a different environment.
  */

 /**
  * Datastore type. This should be 'ICM' for a DB2 Content Manager version 8 datastore and should not be changed. 
  */
 static public final String CM_DATASTORE_TYPE		= "ICM";
 
 /**
  * Name of the DB2 Content Manager library server. The default is 'icmnlsdb'. It needs to be changed
  * if the library server does not have this default name.
  */
 static public final String CM_DATASTORE_NAME		= "icmnlsdb";
 
  /** 
   * If the content object that is routed through the process is of type Folder, the auto link attribute is used to ensure linkage
   * between a folder and its contents.
   * a the value of this attribute can not be changed by the client. Being able to change this value would allow client users to break
   *   the containment relationship between a folder and its members.
   * b if a new folder member is created, the value of its AUTO_LINK_ATTRIBUTE becomes the same as the value of the folder enclosing it
   *   to ensure the member properly autolinks to the enclosing folder.
   */
  static public final String AUTO_LINK_ATTRIBUTE	= "XYZ_ClaimNumber";
  
  /**
   * Helpers.prepareName() uses the following two values to truncate display names to a size that fits the tabular format
   * of the work page.
   * DISPLAY_NAME_SIZE_LIMIT defines when truncation is required (if displayName.length() > DISPLAY_NAME_SIZE_LIMIT)
   * DISPLAY_NAME_TRUNCATION_POS  defines to which size a display name should be truncated if a display name exceeds the limit 
   */
  static public final int DISPLAY_NAME_SIZE_LIMIT		= 50;
  static public final int DISPLAY_NAME_TRUNCATION_POS	= 24;
  
  
  /**
   * Information that is specific to worklist is stored in three arrays: workListStyleMap, addToFolderActionItemTypes, and
   * addToFolderActionNames. Values are retrieved from these arrays based on a work list ID (WLID) that is stored in the HTTP session.
   */
  static public final int ADJUSTER_WLID					= 0;
  static public final int UNDERWRITER_ASSISTANT_WLID	= 1;
  static public final int UNDERWRITER_WLID				= 2;
  static public final int NUM_OF_WLIDS					= 3;
  static public final int DEFAULT_WLID					= 0;
    
  /**
   * This class defines how a user's worklist should look like. The workListStyleMap maps a name of a style name to
   * its definition that specifies what a process staff member sees when logging on to the client.
   */

  public class WorkListQuery {
	public String flowItemTypeName;
	public String flowItemTypeAttributes;
	public String additionalAttributes;
	public String[] columnNames;
	public int[] sortOrder;
	public int posOfStateAttr;

	/**
	 * Define a work list style for a user or group of users.
	 * @param flowItemTypeName name of the item type routed through the process
	 * @param flowItemTypeAttributes attributes of the flow item type that should be retrieved by a worklist query
	 * @param additionalAttributes if the flow item type is a folder this lists the attributes of interest that belong to 
	 * elements of the folder. This is a comma-separated string of values in the form of ItemTypeName/AttributeName wher
	 * ItemTypeName names the item type of a folder member and AttributeName names one of its attributes.
	 * @param columnNames array of names that should be displayed in the first row of the worlist in the sequence in which
	 * they should appear on the screen.
	 * So these names could be derived from the display names of the corresponding attributes, we prefer to define the
	 * column names explicitly here to allow for a more compact representation of the name so each work item fits into a row
	 * @param sortOrder array of positions that maps each entry of the columnNames array to a position in the search result
	 * list. Positions in the search result list are defined as follows:
	 * 1 ... n flowItemTypeAttributes (as defined by the first parameter of this method)
	 * n+1 ... m additionalAttributes (as defined by the second parameter of this method)
	 * m+1 ... m+9 process properties as defined in listWorkItems.jsp (activity instance id, activity name, activity state,
	 * activity owner, work item reason, activation date, process instance name, process name, process instance id) 
	 * @param posOfStateAttr position of the 'actvity state' attribute in the columnNames/sortOrder arrays
	 */

    public WorkListQuery(String flowItemTypeName,
						 String flowItemTypeAttributes,
						 String additionalAttributes,
						 String[] columnNames,
						 int[] sortOrder,
						 int posOfStateAttr)
	{	this.flowItemTypeName = flowItemTypeName;
		this.flowItemTypeAttributes = flowItemTypeAttributes;
		this.additionalAttributes = additionalAttributes;
		this.columnNames = columnNames;
		this.sortOrder = sortOrder;
		this.posOfStateAttr = posOfStateAttr;
	}
	
  }

  /**
   * This array defines the combined queries that specify work lists and correspond to a work list ID.
   */
 
  final static WorkListQuery[] WorkListQueryMap = new WorkListQuery[NUM_OF_WLIDS];
  
  static {
	ProcessData pd = new ProcessData();
	
	WorkListQueryMap[ADJUSTER_WLID] =
	pd.new WorkListQuery(
			"XYZ_ClaimFolder",  // flowItemTypeName
			"XYZ_ClaimNumber",	// flowItemTypeAttributes
			"XYZ_ClaimForm/XYZ_InsrdLName, XYZ_ClaimForm/XYZ_IncDate, XYZ_InsPolicy/XYZ_PolicyNum", //additionalAttributes
			new String[]{ "Date of Incident", "Claim #", "Insured", "Policy #", "Activated", "Work Item State" }, // columnNames
			new int[]   { 11,            	  9,         10,        12,         5,           2},
			5  // posOfStateAttr
			);

	WorkListQueryMap[UNDERWRITER_ASSISTANT_WLID] =
	pd.new WorkListQuery(
			"XYZ_ClaimFolder",  // flowItemTypeName
			"XYZ_ClaimNumber",	// flowItemTypeAttributes
			"XYZ_PolReport/XYZ_ReportNum, XYZ_ClaimForm/XYZ_IncDate, XYZ_ClaimForm/XYZ_InsrdLName, XYZ_AdjReport/XYZ_ClaimAmount", //additionalAttributes
			new String[]{ "Date of Incident", "Claim #", "Insured", "Police Report #", "Estimate ($)", "Activated", "Work Item State" }, // columnNames
			new int[]   { 11,            	  9,         12,        10,                13,             5,           2},
			6  // posOfStateAttr
			);

	WorkListQueryMap[UNDERWRITER_WLID] =
	pd.new WorkListQuery( 
			"XYZ_ClaimFolder",  // flowItemTypeName
			"XYZ_ClaimNumber",	// flowItemTypeAttributes
			"XYZ_PolReport/XYZ_ReportNum, XYZ_ClaimForm/XYZ_IncDate, XYZ_AdjReport/XYZ_ClaimAmount", //additionalAttributes
			new String[]{ "Date of Incident", "Claim #", "Police Report #", "Estimate ($)", "Activated", "Work Item State" }, // columnNames
			new int[]   { 11,            	  9,         10,                12,           	 5,          2},
			5  // posOfStateAttr
			);
    }
  
    /**
    * Map work listIDs to the list of item types that can be added to the folder by this work list.
    * addToFolderActionItemTypes lists the names of item types instances of which can be added to the folder
    * addToFolderActionnames lists the names that should be displayed in the action column as labels for
    * links leading to the corresponding 'add document to folder' operation.
    */

   final static String[][] addToFolderActionItemTypes = { {"XYZ_AdjReport", "XYZ_AutoPhoto"}, {}, {} };
   final static String[][] addToFolderActionNames     = { {"Adjuster Report", "Photo"},				   {}, {} };
 
  
  /**
   * Retrieve a work list query based on a work list ID.
   * @param workListId work list ID used as an index into the WorkListQueryMap array
   * @return corresponding definition of a work list style consisting of flowItemTypeName, flowTypeAttributes, additionalAttributes,
   * array of column names, array of positions, and position of state attribute
   */
  
  static public WorkListQuery getWorkListQuery(int workListId) {
  	return WorkListQueryMap[workListId];
  }
  
  /**
   * Retrieve a list of item types (names) instances of which can be added to the flow folder depending on a work list ID.
   * @param work listId work list ID used as an index into the addToFolderActionNames array
   * @return array of itym type names to be listed in action row of the work page's folder section
   */
  
  static public String[] getAddToFolderActionItemTypes (int workListId) {
  	return addToFolderActionItemTypes[workListId];
  }
  
  /**
   * Retrieve a list of action names to be shown as labels for 'add to folder' operations on a work page
   * @param workListId work list ID used as an index into the addToFolderActionItemTypes array
   * @return array of item type names available to be added to the flow folder for this work list.
   */
  
  static public String[] getAddToFolderActionNames (int workListId) {
	return addToFolderActionNames[workListId];
  }
  
  /**
   * Define the input controls in the process activity section of the work page based on the current activity.
   * Note that instead of drop down boxes a simple input field may be the preferred control. The sample
   * uses drop down boxes to simplify value specification for demo purposes.
   * 
   * @param activityName name of the activity for which corresponding staff action should be returned
   * @param inMsg input message of this activity typically containing parts to be displayed in this HTML section
   * @return corresponding HTML form section with controls for the output message of this activity
   */
  
  static public String staffActions(String activityName,
  							        DataObject inDO) {

  	StringBuffer sb = new StringBuffer();
  	inDO = inDO.getDataObject("request");
	if (activityName.equals("ValidateAndAdjust")) {
	  	String numPolRep = inDO.getString("numPolReports");
	  	String numPhoto  = inDO.getString("numAutoPhotos");
		String contractRetrieved = (inDO.getInt("numContractsRetrieved") == 0) ? "not found" : "retrieved";
		int reminderCount = inDO.getInt("numTimeoutIterations");
		sb.append("<TD colspan=\"4\">\n" +
			"<TABLE>\n" +
				"<TR><TD>Contract " + contractRetrieved + "</TD>" +
				    "<TD>&nbsp;&nbsp;</TD>" +
				    "<TD>Required documents</TD>" +
					"<TD>Adjuster Report</TD>" +
					"<TD><SELECT name=\"msgpart.numAdjReports\">" +
						  "<OPTION value=\"1\" SELECTED>1</OPTION>" +
						"</SELECT>" +
					"</TD>" +
					"<TD>&nbsp;&nbsp;</TD>" +
					"<TD>Remind me in </TD>" +
					"<TD>" +
						"<SELECT name=\"msgpart.timeout\">" +
							"<OPTION value=\"10\" SELECTED>10</OPTION>" +
							"<OPTION value=\"30\">30</OPTION>" +
							"<OPTION value=\"60\">60</OPTION>" +
							"<OPTION value=\"600\">600</OPTION>" +
						"</SELECT> seconds" +
					"</TD>" +
				"</TR>" +
				"</TR>\n" +
				"<TR><TD>Reminder count:" + reminderCount + "</TD>" +
					"<TD colspan=\"2\"></TD>" +
					"<TD>Police Report</TD>" +
					"<TD><SELECT name=\"msgpart.numPolReports\">" +
						  "<OPTION value=\"0\"" + ((numPolRep.equals("0"))?"SELECTED":"") + ">0</OPTION>" +
						  "<OPTION value=\"1\"" + ((numPolRep.equals("1"))?"SELECTED":"") + ">1</OPTION>" +
						  "<OPTION value=\"2\"" + ((numPolRep.equals("2"))?"SELECTED":"") + ">2</OPTION>" +
						  "</SELECT>" +
					"</TD>" +
				"</TR>" +
				"<TR><TD colspan=\"3\"></TD>" +
					"<TD>Auto Photo</TD>" +
					"<TD><SELECT name=\"msgpart.numAutoPhotos\">" +
						  "<OPTION value=\"0\"" + ((numPhoto.equals("0"))?"SELECTED":"") + ">0</OPTION>" +
						  "<OPTION value=\"1\"" + ((numPhoto.equals("1"))?"SELECTED":"") + ">1</OPTION>" +
						  "<OPTION value=\"2\"" + ((numPhoto.equals("2"))?"SELECTED":"") + ">2</OPTION>" +
						  "<OPTION value=\"3\"" + ((numPhoto.equals("3"))?"SELECTED":"") + ">3</OPTION>" +
						  "<OPTION value=\"4\"" + ((numPhoto.equals("4"))?"SELECTED":"") + ">4</OPTION>" +
						  "<OPTION value=\"5\"" + ((numPhoto.equals("5"))?"SELECTED":"") + ">5</OPTION>" +
						"</SELECT>" +
					"</TD>" +
				"</TR>" +
			"</TABLE>" +
			"</TD>");
	}
	else if (activityName.equals("ReviewSmallClaim") || activityName.equals("ReviewLargeClaim")) {
		String fraudHistory = (activityName.equals("ReviewLargeClaim")) ? (String)inDO.getString("fraudHistory") : "";
		sb.append(
			  "<TD colspan=\"2\">\n" +
			  "  <TABLE>\n" +
			  "  <TR>\n" +
			  "    <TD>\n" +
			  fraudHistory +
			  "    </TD>\n" +
			  "    <TD>&nbsp;</TD>\n" +
			  "    <TD><INPUT type=\"radio\" name=\"msgpart.decision\" value=\"approved\" >Approve</TD>\n" +
			  "  </TR>\n" +
			  "  <TR>\n" +
			  "    <TD colspan=\"2\"></TD>\n" +
			  "    <TD><INPUT type=\"radio\" name=\"msgpart.decision\" value=\"rejected\" checked>Reject</TD>\n" +
			  "  </TR>\n" +
			  "  </TABLE>\n" +
			  "</TD>\n" +
			  "<TD colspan=\"2\">\n" +
			  "  Justify decision:<BR>\n" +
			  "  <TEXTAREA name=\"msgpart.comment\" rows=\"2\" cols=\"40\"></TEXTAREA>\n" +
			  "</TD>");
	}
	else  // default: no process data
		sb.append("<TD colspan=\"4\"></TD>");
    return sb.toString();
  }

} // ProcessData
